/*;  element.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl element.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 10:00:32 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/element.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) element.h -- base class of element models

#if !defined(element_h)
#define element_h

#include <stdio.h>
#include <time.h>       //; time_t , tm 

#include "common.h"     //; ץˤ붦
#include "list.h"       //; 饹 List 
#include "object.h"     //; 饹 Object 
#include "receive.h"    //; definition of class Receive_port

//  Definition of the class Element

/*----------------------------------------------------------------------
CLASS

  Element

  abstract class for element models

DESCRIPTION

  An element model

  <ul><li>
    has data members for storing parameters, state
    variables, receive ports, send port,
  <li>
    calculates the time change of the state variables using
    input data obtained through the receive ports,
    and
  <li>
    sends output data through the send ports.
  </ul>

  In hydrolgic simulations, we need various element models because
  various processes are involved.

  Class Element expresses the common part of those hydrologic element
  models.

  <japanese>
  ǥǥ, ѥ᡼, , ü, üҤǡФ˵
  , ѥ᡼̡üҤϥǡѤ, ׻
  Ԥ,̤򹹿, üҤϥǡޤ׻
  ȤΤ, ǥǥ˶̤ʺȤȶ̤ǤʤȤȤʬΥ, 
  ʬΤСʲΤ, ܷǥǥǤΥإåե
  ϡܷǥǥɽ饹Ƥޤ

  ǥǥ뤬ͭü, üҤΥɥ쥹,
  receive_port_array, send_port_array ϿޤܷǥǥǤ,
  Ͽ³˴ؤ뵬ʲԤ, ǥǥκȤδǲʤ
  ȤȤ, ǥǥϿ³˴ؤƱοͿƤޤ
  ޤ, ǤüҤ³ʬϥǥ뤬Ԥޤ, κȤ
  פˤ뤿, ǥǥ뤬ͭüҤ˴ؤ븡ǽ
  .

  ̾ϡǥǥ뤬ξ֤򹹿뤿ɬפʥǡϡüҤ
  ФƼ褦߷פޤʤϡľ̿פˤ
  ơɬפʥǡȤߤ桼Ǥ褦ˡľ̿
  ǤΥɥ쥹򵭲뤿Υǡobject_array Ѱ
  Ƥޤ

  ܷǥǥɽ饹 Element Υǡ, дؿ
  ʲ̤Ǥ

  <pre>
    ǡݥ
        object_list               : ꥹ
        receive_port_array        : üҤΥɥ쥹
        send_port_array           : üҤΥɥ쥹
        object_array              : ľ̿ǤΥɥ쥹
        data_string               : ǡʸ
        current_time              : ǥǥη׻Ѥ߻
        time_step                 : ॹƥå
    дؿ
        Register                  : üҤϿ, ľ̿ǤϿ
        Construct                 : receive_port_array, send_port_array,
                                      object_array κ.
        Get_receive_port          : üҤμ
        Get_send_port             : üҤμ
        Parameter                 : ѥ᡼
        Initial_state             : ̤
        Initial_time              : 
        Calculate_time_step       : ॹƥåפη׻
        Calculate                 : ƥåʬη׻
        Work                      : Calculate 򷫤֤Ѥ, Ϣ
                                      ׻Ԥ

    ============================================================
    added by Y.Ichikawa
    ------------------------------------------------------------
    ǡ
        fixed_time                : ȿ׻ǤǤ˾֤ꤷ
                                      
        target_time               : ȿ׻ɸ
    дؿ
        Fixed_time                : fixed_time ֤ (virtual)
        Target_time               : target_time ֤ (virtual)
        Renew_current_time        : current_time ι (virtual)
        Rewind_current_time       : current_time λ
                                       (virtual)
        Renew_fixed_time          : fixed_time 򹹿 (virtual)
        Renew_target_time         : target_time 򹹿 (virtual)

        Calculate_iteration_step  : ȿ׻ֳִ֤׻ (virtual)
        Do_you_reach_target_time  : target_time ã (virtual)
        Do_you_converge           : «Ƚ (virtual)
        Flush_data                : ȿɸޤǤΥǡν (virtual)
        Send_stocked_data         : ȿɸޤǤΥǡν (virtual)
        Work2                     : ȿ׻ Work 
  </pre>

  ĤδؿϽ貾۴ؿǤ饹 Element 饹Ǥϡ桼
  ʤ餺Ƥ

  <pre>
        ----------------------------------------------------------------
        void Register_receive_ports(void)
        void Register_send_ports(void)

        Register_receive_port, Register_send_port , 饹
        üҥ, üҥФΥɥ쥹 Element::Register ˤ
        Ͽޤ.

        ϿƤʤüҤ, ³ǤʤΤ, ѤǤޤ.

        㤨, 饹Ǽüҥ abc ,
            void Register_receive_port(void) { Register((void*) &abc); }
        Ȥ, abc üҤȤϿǤޤ.

        -----------------------------------------------------------------
        void Set_parameter(FILE* fp)

        ѥ᡼ꤹؿ, 饹˱ޤ.
        եݥ fp ǻꤵե뤫ǡɤߤȤäơ
        ѥ᥿ꤷޤ

        㤨, ѥ᡼ѥǡ p0, p1 ꤹ,

        void Set_parameter(FILE* fp) {
            fscanf(fp, "%lf %lf", &p0, &p1);
        }

        ѥ᥿ľܻꤹؿƱ̾ǡ㤨С

        void Set_parameter(double p0a, double p1a) {
            p0 = p0a;   p1 = p1a;
        }

        Τ褦ѰդƤΤڤǤȻפޤ

        ----------------------------------------------------------------
        void Set_initial_state(FILE* fp)

        ̤ꤹؿ, 饹˱ޤ.
        եݥ fp ǻꤵե뤫ǡɤߤȤäơ
        ̤ꤷޤ

        ǽ֤ե¸ץȤĤĤޤ褦ˤ
        

        

        void Set_initial_state(FILE* fp) {
            fscanf(fp, "%ld %lf", &current_time, &x);
        }

        ǤϡǤ current_time  x ɤߤȤƤޤ

        ----------------------------------------------------------------
        Save_terminal_state(FILE* fp)

        ǽ֤ե˽񤭽Фؿ饹˱ޤ.

        current_time ⤫ʤ餺񤭽ФƤ

        

        Save_terminal_state(FILE* fp) {
            if (fp != NULL) {
               fprintf(fp, "%ld %lf", current_time, x);
            }
        }

        ----------------------------------------------------------------
        Boolean Initial_output(void)

        ̤᤿ľˡcurrent_time ǤνϤޤ
        üҤ˻Ƚͤ񤭹ߤޤcurrnet_time ǤνϤ
        Ǥ YES ֤ޤɬפϥǡƤʤ
        ʤɤơcurrent_time νϤ׻ǤʤäϡNO ֤
        ޤ

        

        sp.Send_data(current_time, s0);
        retune YES;

        ǹԤʤϤϡѥ᥿ȽͤӼüҤ
        Ȥ׻Ǥ褦ˤʤäƤʤȤޤcurrent_time 
        νϤ׻ǤϡYES ֤褦ˤޤɬפʥǡ
        ʤơcurrent_time ǤνϤ׻ǤʤäϡNO
        ֤Ƥ

        ʬϥǥηϥǥǤϡInitial_output ϡºݤˤϲ
        ʤ褦Ƥ⹽ޤ. ºݡʬϥǥηϥ
         Initial_output ΥǥեȤϡ⤻ YES 
        ޤ

        ǥǥǤϡɬcurrent_time ǤΥǡ
        褦Ƥ


        ----------------------------------------------------------------
        time_t Calculate_time_step(void)

        饹ǥǥ˱ơ η׻Υॹƥå
        time_step (sec) ׻, ͤȤ֤ؿƲ.

        Ĥޤ, time_step ɤΤ褦ʾﲼ, ɤΤ褦ͤˤ뤫
        ޤ.

        㤨, time_step  3600 (sec) ꤹ,
            time_t Calculate_time_step(void) { return 3600; }
        Ʋ.

        ޤΤ褦, ǡ k ͤˤ time_step ѹ
        ȤǤޤ.
            time_t Calculate_time_step(void) { return (k > 0) ? 360 : 60; }
            time_t Calculate_time_step(void) { return 360 * pow(10, k); }

        ǡ Element::time_step , Element::Work Ǥδؿ
        ͤʤɤȤäꤷޤ
        Τᡤ㤨С
            time_t Calculate_time_step(void) { time_step = 1; return 1; }
        Τ褦, δؿ time_step ľѹʤǤ

        Calculate_time_step , дؿ Work ΥץȤѰդ
        Ƥ Work0 ǸƤФƤޤWork1 Ȥ, ȼ
        Work ˤ, Calculate_time_step ƤФʤȤ
        ޤϡ

        time_t Calulate_time_step(void) { return 0; }

        ʤɤصŪ򤷤ƤƤ

        ----------------------------------------------------------------
        Boolean Can_you_calculate(void)
        ǡǤƷ׻ǤݤȽ

        ƤνͭüҤ Necessary_time_from, Necessary_time_to
        ֤ΥǡѥåǽǤ뤳Ȥǧޤ.
        ǽʾ, ׻¹ԲǽȽǤ YES ֤ޤ.
        Բǽʾ, ׻¹ԲĤȽǤ NO ֤ޤ.

        ǡԲǽǤֻ򤷤ƤüҤФơ
        ׻׵ФޤŪˤϡüҤ Creq å
        ޤ(2000-09-13 յ)

        RETURN:
            YES   ׻¹Բǽξ.
            NO    ׻¹Բǽξ.
                    ׻Ѥߤξ.

        桼ʬ, Can_you_calculate Ƥ⹽ޤ,
        ץȤ, ĤΣĤδؿѰդƤޤ

        Can_you_calculate0(void)

        Element Υдؿ Necessary_time_from, Necessary_time_to 
        ֤δ֤ΥǡǤз׻ǽȽǤ롣
        ֤ΤǤNecessary_time_from, Necessary_time_to ؿϡ
        桼ͳ˺ǤޤǥեȤǤϡ
        Necessary_time_from  current_time ֤Necessary_time_to ϡ
        current_time + time_step ֤ͤޤ

        ǡԲǽǤֻ򤷤ƤüҤФơ
        ׻׵ФޤŪˤϡüҤ Creq å
        ޤ(2000-09-13 յ)

        Can_you_calculate1(void)

        Necessary_time_from ǡǤз׻ǽȽ
        Ǥޤ(ξϡNecessary_time_to ؿ̣Ϥ
        ޤ)

        ǡԲǽǤֻ򤷤ƤüҤФơ
        ׻׵ФޤŪˤϡüҤ Creq å
        ޤ(2000-09-13 յ)

        Ȥ, Can_you_calculate0(void) ֤ʤ,

        Boolean Can_you_calculate(void) { return Can_you_calculate0(); }

        Ф褤Ǥ

        ----------------------------------------------------------------
        Boolean Calculate(void)

        ƥåʬη׻³Ʋ.

        Ū Element Ǥ,
            ǽ, ƾ̤ˤ, current_time ˤͤǼƤ.
            Ǥ current_time + time_step ˤ, ͤ
              ׻.
            δؿλ, ̤ current_time + time_step ˤ
              ͤ˹Ƥ.
        Σ­褦,  (0)  (4) ˵ҤƲ.

            (1) üҤ.
            (2) ׻.
            (3) üҤؤν.
            (4) ̤ι.

        (2) Ǥ, ѥ᡼, current_time ˤ, üҤ
        ϤѤ, current_time + time_step ˤ, ٤
        ͤ׻, ǥΣƥåʬŪƲ.

        Υдؿ, ƥåפη׻Ǥ, YES ֤ޤ
        ׻ʤǷ׻Ǥʤʤä, NO ֤ޤ

        δؿ current_time, time_step ѹʤǲ.
        ɬפʤ time_step 򹹤˺٤ֶʬʬ䤷Ʒ׻Ǥޤ,
        λϺѿѤ, current_time, time_step ѹʤ
        դƲ.

        ׻ɬפƤοͤ, ǡ, ڤӽͭüҤ
        ϤȤƼǤޤ. äưפǤ.

        貾ؿʤΤ, 饹ɬƲ.

        ACCIDENT:
            δؿ current_time, time_step ѹ줿, 
              ȯζ줬.

        ----------------------------------------------------------------
        Boolean Work(void)
        current_time, Terminal_time ֤η׻

        Calculate, Calculate_time_step ʤɤѤ, Terminal_time ޤǤ
        ׻¹Ԥޤ.

        ޤ, Element::Can_you_calculate ͤ NO ʤ, ׻
        ĤȽǤ, NO ֤ޤ.

        ͤ YES ξ, Element::Can_you_calculate ͤ YES
        Ǥ¤, Element::Calculate ¹Ԥcurrent_time +=
        time_step 򷫤֤¹Ԥޤ.

        Ǹ, Element::Set_data_pack_array, Element::Report_to_sender
        ¹Ԥ, YES ֤ޤ.

        RETURN:
            YES   ׻¹Ԥ.
            NO    ׻Բǽξ.
                    ׻Ѥߤξ.
        ----------------------------------------------------------------

   </pre>

   δؿȿ׻ԤʤǤɬƲǤʤȡ
   顼åФ exit Ƥޤޤ

   <pre>
        --------------------------------------------------------
        time_t Calculate_iteration_step(void)
        ȿ׻ֳִ֤η׻

        饹ǥǥ˱ơȿ׻Υॹƥå
        ׻ͤȤ֤ؿƲ

        δؿǤ OhymosOption ΤȤͤɬפϤޤ
        OhymosOption  Renew_target_time ǹθޤ

        --------------------------------------------------------
        Boolean Element::Do_you_converge(void)
        ȿ׻μ«Ƚ

        饹ǥǥ˱ơȿ׻μ«ȽꤷƲ
        «Ƥ YES«Ƥʤ NO ֤
        

        --------------------------------------------------------
        void Element::Send_stocked_data(void)
        fixed_time  target_time ޤǤΥǡν

        ȿ׻ǤǤϡ̡׻«Ƥǡ
        ϤޤäơǤ fixed_time  target_time
        ޤǤ줿ǡŪݴɤ«ǧ줿
        ǽϤޤ«ʤäΥǡνˤĤ
        Ƥϡ桼ˤޤޤŪˤϡɬפȤʤ
        ǡ Rewind_current_time ǽʬΤ褤Ȼפ
        ޤ

        --------------------------------------------------------
  </pre>
  </japanese>

----------------------------------------------------------------------*/

class Element : public Object {

public:

  //////////////////////////////////////////////////////////////////////
  // OperationFlag
  //
  // [] E_UNDONE   initial work could not be done.
  // [] E_DONE     initial work was completed.
  // [] E_ALREADY  initial work had been done already.
  //
  // This type of variable is used in Do_initial_work(void).
  enum OperationFlag {E_UNDONE, E_DONE, E_ALREADY};
    //; E_UNDONE:   Ȥ򤹤뤳ȤǤʤä
    //; E_DONE:     Ȥ¹Ԥ
    //; E_ALREADY:   ȤϤǤ˼¹ԺѤǤä

//; 0.  ǡ.
protected:

  //////////////////////////////////////////////////////////////////////
  // whether the initial work has been completed or not.
  //
  // <japanese>
  // Initial_work 򽪤ä (YES) ݤ (NO)
  // </japanese>
  Boolean initialworkdone_p;

  //////////////////////////////////////////////////////////////////////
  // whether the 'Share_info' has been completed or not.
  //
  // <japanese>
  // Share_info 򽪤(YES)ݤ(NO)
  // </japanese>
  Boolean shareinfodone_p;

  //////////////////////////////////////////////////////////////////////
  // pointer to a list for storing Object*
  //
  // <japanese>
  // Object* Υꥹȡreceive_port_array, send_port_array κ˺
  // ѤȤƻѤޤ
  // </japanese>
  List* object_list;

  //////////////////////////////////////////////////////////////////////
  // array of Receive_port*
  //
  // <japanese>
  // üҤؤΥݥ󥿤
  // </japanese>
  void** receive_port_array;

  //////////////////////////////////////////////////////////////////////
  // number of receive ports
  //
  // <japanese>
  // receive_port_array ǿ
  // </japanese>
  int number_of_receive_ports;

  //////////////////////////////////////////////////////////////////////
  // array of Send_port*
  //
  // <japanese>
  // ü
  // </japanese>
  void** send_port_array;

  //////////////////////////////////////////////////////////////////////
  // number of send-ports
  //
  // <japanese>
  // send_port_array ǿ
  // </japanese>
  int number_of_send_ports;

  //////////////////////////////////////////////////////////////////////
  // array of Object*
  //
  // <japanese>
  // ֥
  // </japanese>
  void** object_array;

  //////////////////////////////////////////////////////////////////////
  // the number of objects stored in object_array
  //
  // <japanese>
  // object_array ǿ
  // </japanese>
  int number_of_objects;

  //////////////////////////////////////////////////////////////////////
  // data string
  //
  // <japanese>
  // ǡʸ
  // </japanese>
  char* data_string;

  //////////////////////////////////////////////////////////////////////
  // current time in unix time (sec)
  //
  // <japanese>
  //  (׻Ѥ)  (sec)
  // </japanese>
  time_t current_time;

  //////////////////////////////////////////////////////////////////////
  // time step (sec)
  //
  // <japanese>
  // ׻ƥåפλ (sec)
  // </japanese>
  time_t time_step;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ȿ׻ˤƾ֤ꤷƤ (sec)
  // </japanese>
  time_t fixed_time;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ȿ׻ɸ (sec)
  //
  // (*  *)
  // ȿ׻ǤǤϡ׻ϻ fixed_time,
  // target_time  current_time Ȱפɬפޤ
  // Set_initial_state  current_time ꤵ줿塢
  //  void Cur2fix2tar(void) ֤ľ
  // target_time = fixed_time = current_time; ȤƤ
  // </japanese>
  time_t target_time;

        /* <japanese>
        data_string , ѥ᡼, ̤ѥݥ󥿤Ǥ.

        Element  owner ˤ Sub_system ֥Ȥꤵޤ.
        </japanese> */

  //////////////////////////////////////////////////////////////////////
  // Flag of request of calculation. Initial value is NO.
  //
  // <japanese>
  // ή¦׻褦ȤΤ˷׻ǤʤǤˤȤˡ
  // mCreqp = YES ꤵ褦ˤޤơElement ϡ mCreqp
  // = YES ΤȤϡ׻褦Ϥޤ
  // mCreqp ϡclass Element ǻϤɲä줿ǡФǤclass
  // Element Υ֥ȤäȤˤϡmCreqp  NO ˽
  // 
  // </japanese>
  Boolean mCreqp;

//; 1.  receive_port_array, send_port_array, object_array κ, .
protected:

  //////////////////////////////////////////////////////////////////////
  // Finds an object with class name 'c_name', object name 'o_name', and
  // object number 'o_num' from object_list and returns the pointer to
  // the object.  When such an object could not be found from
  // object_list, NULL will be returned.
  //
  // <japanese>
  // object_list  Object ֥Ȥμ
  //
  // Get_from_list , 饹̾, ֥̾, ֹ֥
  // , object_list  Object ֥ȤΥɥ쥹ޤ.
  //
  // ꤵ줿֥ȤΥɥ쥹ϿƤʤ, NULL 
  // ֤ޤ.
  //
  // object_list  NULL ξ, 顼åɽ, NULL 
  // ޤ.
  //
  // ARGUMENT:
  //   c_name        륪֥ȤΥ饹̾.
  //   o_name        륪֥ȤΥ֥̾.
  //   o_num         륪֥ȤΥֹ֥.
  // RETURN:
  //   void*         void ˥㥹Ȥ줿ꥪ֥ȤΥ
  //                   쥹.
  //   NULL          ꥪ֥ȤȯǤʤ.
  //                   object_list ¸ߤʤ.
  // ACCIDENT:
  //   Object* ʳѿ object_list ˼ϿƤ
  //                   .
  //
  // </japanese>
  void* Get_from_list(char* c_name, char* o_name, int o_num);

  //////////////////////////////////////////////////////////////////////
  // Finds an object with object name 'o_name', and
  // object number 'o_num' from object_list and returns the pointer to
  // the object.  When such an object could not be found from
  // object_list, NULL will be returned.
  void* Get_from_list(char* o_name, int o_num);

protected:

  //////////////////////////////////////////////////////////////////////
  // Registers an object 'obj' to the data member 'object_list'
  //
  // <japanese>
  // Object ֥ȤϿ
  //
  // Object ֥Ȥ object_list Ͽޤ.
  //
  // Ͽˤ, ξϥ顼åϸ NO ֤ޤ.
  //
  // <pre>
  //      obj  NULL ξ.
  //      object_list  NULL ξ.
  //      饹̾, ֥̾, ֹ֥椬 obj Ȱ
  //        륪֥Ȥ object_list ϿƤ.
  //      obj  owner Ѥߤξ.
  // </pre>
  //
  // List::Put_tail  object_list Τεΰ褬ݤǤʤ
  // , 顼åϤ, ٥룵Ƕλޤ.
  //
  // ޤ, ξϰ۾ȯޤ.
  //
  // <pre>
  //      obj  Object* ʳѿǼƤ.
  //      object_list  Object* ʳѿϿƤ
  //        .
  //
  // ARGUMENT:
  //     obj       void* ˥㥹Ȥ줿Ͽ֥ȤΥɥ
  //                 .
  // RETURN:
  //     YES       Ͽ줿.
  //     NO        obj  NULL ξ.
  //                 object_list  NULL ξ. (󲽺Ѥߡ)
  //                 obj , 饹̾, ֥̾, ֥
  //                 椬ưפ륪֥Ȥ object_list 
  //                 ϿƤ.
  //                 obj  owner Ѥߤξ.
  //                 (Object::Set_owner)
  // EXIT:
  //     Level 5   object_list  obj Ͽ뤿Υݤ
  //                 ʤ. (List::Put_tail)
  // ACCIDENT:
  //     obj  Object* ʳѿǼƤ.
  //     object_list  Object* ʳѿϿƤ
  //       .
  // </pre>
  // </japanese>
  Boolean Register(void* obj);

protected:
  //////////////////////////////////////////////////////////////////////
  // Declares member function Register_receive_ports(void) as pure
  // virtual.  In classes derived from class Element, its definition
  // should be given.
  //
  // <japanese>
  // Register_receive_port , 貾۴ؿǤ.
  // 饹ɬƲ.
  //
  // Register_receive_port , 饹
  // üҥ, üҥФΥɥ쥹 Element::Register ˤ
  // Ͽޤ.
  //
  // ϿƤʤüҤ, ³ǤʤΤ, ѤǤޤ.
  //
  // 㤨, 饹Ǽüҥ abc ,
  // <dl><dd>
  // void Register_receive_port(void) { Register((void*) &abc); }
  // </dl>
  //Ȥ, abc üҤȤϿǤޤ.
  // </japanese>
  virtual void Register_receive_ports(void) = 0;

  //////////////////////////////////////////////////////////////////////
  // Declares member function Register_send_ports(void) as pure
  // virtual.  In classes derived from class Element, its definition
  // should be given.
  //
  // <japanese>
  // Register_send_port , 貾۴ؿǤ.
  // 饹ɬƲ.
  //
  // Register_send_port , 饹
  // üҥ, üҥФΥɥ쥹 Element::Register ˤ
  // Ͽޤ.
  //
  // ϿƤʤüҤ, ³ǤʤΤ, ѤǤޤ.
  //
  // 㤨, 饹Ǽüҥ abc ,
  // <dl><dd>
  // void Register_send_port(void) { Register((void*) &abc); }
  // </dl>
  //Ȥ, abc üҤȤϿǤޤ.
  // </japanese>
  virtual void Register_send_ports(void) = 0;

protected:
  //////////////////////////////////////////////////////////////////////
  // Creates an array of Objects* which are included in 'objext_list'
  // and stores its address in 'obj_array' and the number of elements in
  // 'num_of_objs'
  //
  // <japanese>
  // object_list 󲽤 obj_array ꤷ, ǿ num_of_objs
  // ꤷޤ.
  //
  // object_list  NULL ؤƤ, 顼åɽ
  // NO ֤ޤ.
  //
  // ޤ num_of_objs  0 ʾξ, ѤߤȽǤ,
  // 顼åɽ NO ֤ޤ.
  // 󲽸, num_of_objs  0 ʾꤵޤ.
  // ä, num_of_objs  0 ̤˽ƤƲ.
  //
  // number_of_receive_port, number_of_send_port , 󥹥ȥ饯
  // -1 ˽ޤ.
  //
  // [in] obj_array -  Υݥ󥿤Ǽ뻲Ȱ.
  // [in] num_of_objs -  obj_array ǿǼ뻲Ȱ.
  //
  // [RETURN] YES -  obj_array 줿.
  // [RETURN] NO  - object_list  NULL ξ䡢num_of_objs  0 ʾ
  //                ξ.
  //
  // [EXIT] Level5 - obj_array εΰݤǤʤ.
  // </japanese>
  Boolean Make_object_array(void**& obj_array, int& num_of_objs);

protected:
  //////////////////////////////////////////////////////////////////////
  // Acquire the memory to store 'num_of_objs' objects and makes
  // 'object_array' point to the memory
  //
  // <japanese>
  // object_array κ
  //
  // num_of_obj ĤΥ֥ȤΥɥ쥹򵭲뤿ΰҡ
  // פ˳ݤobject_array Ƭɥ쥹ؤ褦˥åȤ
  // ޤ
  //
  // object_array ľ̿򤹤Υɥ쥹򵭲뤿˻Ȥ
  // ޤ
  //
  // ξϥ顼åϸ NO ֤ޤ.
  // <pre>
  //     num_of_objs ξ.
  //     number_of_objects  0 ۾ξ. (Ѥ)
  // </pre>
  //
  // ޤ object_array εΰݤǤʤ, 顼å
  // ϸ٥룵Ƕλޤ
  //
  // <pre>
  // ARGUMENT:
  //     num_of_objs   object_array ˵륪֥ȿ.
  // RETURN:
  //     YES           object_array ݤǤ.
  //     NO            num_of_objs ξ.
  //                     number_of_objects  0 ۾ξ. (Ѥ)
  // EXIT:
  //     object_array εΰ褬ݤǤʤ. (new)
  // </pre>
  // </japanese>
  Boolean Set_object_array(int num_of_objs);

public:
  //////////////////////////////////////////////////////////////////////
  // Registers the object 'obj' with which the current object
  // communicates directly to object_array[num]
  //
  // <japanese>
  // Object ֥ȤϿ
  //
  // ľ̿Υɥ쥹 obj  object_array[num] ˵ޤ.
  //
  // , ξϥ顼åϸ NO ֤ޤ.
  // <pre>
  //     num ξ.
  //     num  number_of_objects ʾξ.
  //     obj  NULL ξ.
  //     object_array[num] ̤Υ֥ȤƤ
  //       .
  //
  // ARGUMENT:
  //     obj   void* ˥㥹Ȥ줿Ͽ֥ȤΥɥ쥹.
  //     num   object_array[num]  obj 򵭲.
  // RETURN:
  //     YES   object_array[num]  obj 줿.
  //     NO    num ξ.
  //             num  number_of_objects ʾξ.
  //             obj  NULL ξ.
  //             object_array[num] ̤Υ֥ȤѤߤξ.
  // </pre>
  // </japanese>
  Boolean Register(void* obj, int num);

public:
  //////////////////////////////////////////////////////////////////////
  // Creates 'receive_port_array' and 'send_port_array'.
  //
  // Construct does the following work
  //
  // <ol>
  // <li> Creates a List object in the heap area and make 'object_list'
  //      point to the List object,
  // <li> Creates 'receive_port_array' by the use of
  //      'Register_receive_port' and 'Make_object_array',
  // <li> Reset 'object_list' by the use of List::clear,
  // <li> Creates 'send_port_array' by the use of
  //      'Register_send_port' and 'Make_object_array', and
  // <li> Deletes 'object_list' and set it to NULL
  // </ol>
  //
  // <japanese>
  // Object ֥ȵκ
  //
  // Object ֥ȵޤ.
  //
  // ޤ object_list  NULL Ǥ뤳Ȥǧޤ. NULL Ǥʤ,
  // object_list ȽǤƥ顼åɽ, return 
  // ޤ.
  //
  // , ʲκȤ¹Ԥޤ.
  // <pre>
  //     List ֥Ȥҡפ˺ object_list ꤹ.
  //     Element::Register_receive_port, Elenment::Make_object_array
  //       Ѥ receive_port_array .
  //     List::Clear ˤ object_list .
  //     Element::Register_send_port, Element::Make_object_array 
  //       Ѥ send_port_array .
  // </pre>
  //
  // Ǹ object_list  delete , NULL ꤷޤ.
  //
  // Make_object_array ϲ۴ؿʤΤ, 饹ѹǤޤ.
  //
  // </japanese>
  virtual Boolean Construct(void);

protected:
// modified by Y.Ichikawa (Wed Dec 27 16:07:51 1995)
  //////////////////////////////////////////////////////////////////////
  void* Get_from_array(char* c_name, char* o_name, int o_num,
                         void** obj_array, int num_of_objs);
  //////////////////////////////////////////////////////////////////////
  void* Get_from_array(char* o_name, int o_num,
                         void** obj_array, int num_of_objs);
public:

//; modified by Y.Ichikawa (Wed Dec 27 16:07:51 1995)
//; iterator_array Ƴȼä Get_from_array ѹ줿ᡢ
//; ʲθޤĤδؿѹ

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // 饹̾, ֥̾, ֹ֥ꤷ,
  // receive_port_array , üҤΥɥ쥹ޤ.
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  void* Get_receive_port(char* c_name, char* o_name,
    int o_num = 0);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // 饹̾, ֥̾, ֹ֥ꤷ,
  // send_port_array , üҤΥɥ쥹ޤ.
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  void* Get_send_port(char* c_name, char* o_name, int o_num = 0);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // üҤμ
  //
  // ֥̾, ֹ֥ꤷ, receive_port_array
  // , üҤΥɥ쥹ޤ.
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  void* Get_receive_port(char* o_name, int o_num = 0);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // üҤμ
  //
  // ֥̾, ֹ֥ꤷ, send_port_array
  // , üҤΥɥ쥹ޤ.
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  void* Get_send_port(char* o_name, int o_num = 0);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥Ȥμ
  //
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  void* Get_object(char* c_name, char* o_name, int o_num = 0);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥Ȥμ
  //
  // 饹̾, ֥̾, ֹ֥ꤷ,
  // object_array , ֥ȤΥɥ쥹ޤ.
  // ꤵ줿֥ȤؤΥݥ󥿤ϿƤʤ,
  // NULL ֤ޤ.
  // </japanese>
  virtual void* Get_component(char* c_name, char* o_name, int o_num = 0)
        {
        Eprint(__FILE__, __LINE__);
            // Element::Get_component(char*,char*,int)
            // This is 'element'. Has no 'component'.
        fprintf(stderr,
            "\tc_name(arg) = %s, o_name(arg) = %s, o_num(arg) = %d\n",
            c_name, o_name, o_num);
        Oprint();
        return NULL;
        }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥̾, ֹ֥ꤷ, ǤΥɥ
  // ֤ޤ
  // </japanese>
  virtual void* Get_component(char* o_name, int o_num = 0)
        {
        Eprint(__FILE__, __LINE__);
            // Element::Get_component(char*,int)
            // This is 'element'. Has no 'component'.
        fprintf(stderr,
            "\to_name(arg) = %s, o_num(arg) = %d\n",
            o_name, o_num);
        Oprint();
        return NULL;
        }

//; 2.  ³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  //
  // ƤνͭüҤ Send_port::Switch ¹Ԥޤ.
  //
  // λ, Send_port::buffer_list üϿѤ, ǡ
  // ѥåϿѤž, ü, ǡѥå
  // ޤ.
  //
  // δؿμ¹Ը, ͭüҤ³, 󥯤Ǥޤ.
  // <pre>
  //    RETURN:
  //        YES   ˼¹ԤǤ.
  //        NO    Send_port::Switch  NO ֤üҤ¸ߤ.
  //                ( Send_port::Switch Ƥ. )
  //    EXIT:
  //        Level 5   Send_port::Switch Ƕλ.
  // </pre>
  // </japanese>
  virtual Boolean Switch(void);

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ׻γǧ
  //
  // ü, üҤ, ǡѥåǽʾ֤Ǥ뤳Ȥ
  // ǧޤ.
  //
  // üҤ Receive_port::Are_you_ready , ڤü
  //  Send_port::Are_you_ready ͤ YES Ǥ뤳Ȥǧޤ.
  // ͤ NO üҤ¸ߤ NO ֤, ¸ߤʤ YES
  // ֤ޤ.
  //
  // δؿͤ YES ξ, Element Ϸ׻¹ԤǤޤ.
  //
  // NO ξ, ǡѥåǤʤü, ޤϥǡѥ
  // ǤʤüҤ¸ߤ,  Element,  Element
  // Ƿ׻¹ԤǤޤ.
  //
  // ξ, ³Ȥ˸꤬뤫, Swith_send_port ¹ԤƤ
  // ǽޤ.
  // <pre>
  //     RETURN:
  //      YES   üҤ Are_you_ready ͤ YES ξ.
  //      NO    ͤ NO üҤ¸ߤ.
  // </pre>
  // </japanese>
  virtual Boolean Are_you_ready(void);

//; 3.  ͤν
public:
  //////////////////////////////////////////////////////////////////////
  Boolean Set_data_string(char* d_str, int d_strlen = -1);

  //////////////////////////////////////////////////////////////////////
  virtual void Clear_data_string(void);

  //////////////////////////////////////////////////////////////////////
  Boolean Get_data_string(char* d_str);

protected:
  //////////////////////////////////////////////////////////////////////
  Boolean Look_data_string(char*& buffer);
        /* <japanese>
          Element::Set_data_string(char*,int)     ǡʸ
          Element::Clear_data_string(void) (virtual)
                ǡʸξõ
          Element::Get_data_string(char*)
                ǡʸ񤭹ΰμ
          Element::Look_data_string(char*) (protected)
                ʣǡʸμ


        Լʬ򻲾Ȥ... => element.cc


        data_string λؤ, Τ褦 '\0' Ƕڤ줿ʣΥ
        ʸ [data_0], [data_1], ... , [data_n] ǼƤ,
            [data_0] \0 [data_1] \0 ... [data_n] \0 \0
        [data_0], [data_1], ... , [data_n] ֤˼ޤ.

        ޤ buffer  data_string ƽ, θ, δؿ
        ֤Ѥ, ǡʸƬ֤ buffer ˼ޤ.
        buffer ˥ǡʸǤ YES ֤ޤ.
        ƤΥǡʸѤߤλ, buffer  '\0' Ǽ NO 
        ֤ޤ.

        㤨, Τ褦ѤƲ.
            buffer = data_string;
            for (int k = 0; k < xxx; k++) {
                sscanf(buffer,
                    "%d %d ... %d",
                    &member_1[k], &member_2[k], ..., &member_n[k]);
                if (Look_data_string(buffer) == NO) break;
            }

        data_string  NULL ξ, 顼åϸ NO ֤ޤ.

        ARGUMENT:
            buffer    ǡʸǼ뻲Ȱ.
        RETURN:
            YES       ǡʸ󤬼Ǥ.
            NO        ǡʸμλ.
                        data_string  NULL ξ.
        </japanese> */
protected:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // stock_p  NO ΤȤϡfp  1 ɤߤȤ, buffer ˳Ǽޤ
  // 裱 # ԤԤɤФޤƬʸ
  // iflag ˳Ǽޤstock_p == YES ξϡºݤˤɤߤȤ餺
  // ߤȤä򤷤ơޤ stock_p  NO ѹޤǡ
  // ɤᤷ¸ƤޤɤߤȤ뤳ȤǤʤä,
  // NULL ֤ޤɤߤȤ줿, buffer ֤ޤ
  //
  // [in,out] fp      - file pointer to the file to be read in
  // [in,out] buffer  - buffer
  // [in,out] stock_p - NO: no stock, YES: previous line is stocked
  // [out] iflag      - first non blank character
  // </japanese>
  char* Get_line(FILE* fp, char* buffer, int buflen,
                   Boolean& stock_p, int& iflag);

  //////////////////////////////////////////////////////////////////////
  void Get_line(FILE*fp, char* buffer, int buflen, char* fname,
                   int line_number);
        /* <japanese>
          Element::Get_line(FILE* fp, char*& buffer, int buflen,
        char* fname, int line_number);

        Read a line from the file "fp" and store the line into
        "buffer". The lines which start with "#" or contain only white
        characters are regarded as comment lines and skipped.

        "buflen" and "line_number" must contain the name of the file and
        the line number which call this member funcion.

        For example, call as
        Get_line(fp, buffer, BUFFLEN, __FILE__, __LINE__);

        </japanese> */

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // եݥ fp ؤե뤫ѥ᥿ɤ߼ޤ
  // ۴ؿǤǥǥץޤǥǥ˱
  // ɬפޤ
  //
  // եݥ fp ǻꤵե뤫ǡɤߤȤäơ
  // ѥ᥿ꤷޤ
  //
  // 㤨, ѥ᡼ѥǡ p0, p1 ꤹ,
  //
  // <pre>
  //     void Set_parameter(FILE* fp) {
  //        fscanf(fp, "%lf %lf", &p0, &p1);
  //     }
  // </pre>
  // ѥ᥿ľܻꤹؿƱ̾ǡ㤨С
  // <pre>
  //      void Set_parameter(double p0a, double p1a) {
  //          p0 = p0a;   p1 = p1a;
  //        }
  // </pre>
  // Τ褦ѰդƤΤڤǤȻפޤ
  //
  // 貾۴ؿʤΤ, 饹ɬƲ.
  // </japanese>
  virtual void Set_parameter(FILE* fp) = 0;

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ѥ᥿ɤ߹ؿѤߤǡ桼ϿɬפϤ
  // ޤ
  //
  // </japanese>
  virtual void Parameter(FILE* fp, char*& buffer, int buflen,
        Boolean& stock_p);

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  //
  // ̤ꤹؿ, 饹˱ޤ.
  // եݥ fp ǻꤵե뤫ǡɤߤȤäơ
  // ̤ꤷޤ
  //
  // ǽ֤ե¸ץȤĤĤޤ褦ˤ
  // 
  //
  // 
  // <pre>
  //      void Set_initial_state(FILE* fp) {
  //          fscanf(fp, "%ld %lf", &current_time, &x);
  //      }
  // </pre>
  // ǤϡǤ current_time  x ɤߤȤƤޤ
  //
  // Set_initial_state Ͻ貾۴ؿʤΤ, 饹ɬ
  // .
  // </japanese>
  virtual void Set_initial_state(FILE* fp) = 0;

  //////////////////////////////////////////////////////////////////////
  virtual void Initial_state(FILE* fp, char*& buffer, int buflen,
        Boolean& stock_p);

// added by Yutaka ICHIKAWA (Sun Dec  1 18:05:10 1996)
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Share_info ϡľ̿Ѥƥѥ᥿ʤɤξͭ뤿
  // ؿǤ͡ˡͤޤȤФ륪֥
  // Υѥ᥿¾Υ֥ȤǤѤʤɤѤǤޤ
  // 줾줬Ʊѥ᥿ Set_parameter ꤹΤǤϤʤ
  // ꤷѥ᥿ͤ¾ԡƤ餦뤤ϤΥѥ
  // ؤΥݥ󥿤򶵤Ƥ餦ʤɤȤäȤͤޤ
  // ͭȤԤä YES ֤ƲȤԤʤ
  //  NO ֤ƲǥեȤǤñ YES ֤褦
  // Ƥޤ
  // </japanese>
  virtual Boolean Share_info(void) { return YES; }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Do_share_info  Share_info ƤӽФؿǤ
  // Do_share_info  Share_info ƤӽФؿǤ
  //  Element::OperationFlag Ǥ
  // Do_share_info  Share_info ޤʤƤʤ :
  //
  // <ol>
  //   <li> Share_info Ƥӡͤ YES ʤ shareinfodone_p
  //         YES ꤷơE_DONE ֤ޤ
  //
  //   <li> Share_info ͤ NO ʤ ͭȤǤ
  //        äȤ E_UNDONE ֤ޤ
  // </ol>
  //
  // Share_info ǤˤǤ E_ALREADY ֤ޤ
  // </japanese>
  virtual Element::OperationFlag Do_share_info(void);

  /* <japanese>

     </japanese>
   */

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Ǥ³Ƥ뤹٤ƤüҤΥǡ˼ϿƤǡ
  // ѥå, եݥ fp ǻꤵƤե˽񤭽
  // ޤϿλŻ, üҤƱ̾Υдؿ
  // Send_port::Save_send_port_stock ȤäƹԤʤޤ
  //
  // Sub_system Ǥ, ޤ
  // </japanese>
  virtual void Save_send_port_stock(FILE* fp);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ǡʸ data_string ɤߤȤ, Ǥ³Ƥü
  // Ҥ˥ǡϿޤ
  //
  // </japanese>
  void Restore_send_port_stock(void);

//; 4.  ߥ졼˴ؤ.
public:
  //////////////////////////////////////////////////////////////////////
  time_t Current_time(void) { return current_time; }
  //////////////////////////////////////////////////////////////////////
  time_t Time_step(void) { return time_step; }
  //////////////////////////////////////////////////////////////////////
  time_t Next_time(void) { return (current_time + time_step); }
  //////////////////////////////////////////////////////////////////////
  virtual time_t Terminal_time(void)
        { return ((Element*) owner)->Terminal_time(); }

  //////////////////////////////////////////////////////////////////////
  // Returns the medium term target time
  //
  // <japanese>
  // ɸowner  get ޤ
  // </japanese>
  virtual time_t MediumTermTargetTime(void)
  { return ((Element*)owner)->MediumTermTargetTime();}

    // added by Y.Ichikawa (Fri Nov 10 15:01:38 1995)
public:
  //////////////////////////////////////////////////////////////////////
  time_t Fixed_time(void) { return fixed_time; }
  //////////////////////////////////////////////////////////////////////
  time_t Target_time(void) { return target_time; }
protected:
  //////////////////////////////////////////////////////////////////////
  void Cur2fix2tar(void) { target_time = fixed_time = current_time; }

        /* <japanese>
          Element::Current_time(void) (inline)    current_time ֤
          Element::Time_step(void) (inline)       time_step ֤
          Element::Next_time(void) (inline)
                ƥåפ current_time ֤
          Element::Terminal_time(void) (inline,virtual)
                ׻λ֤
          Element::Fixed_time(void) (inline)
                ֳ֤
          Element::Target_time(void) (inline)
                ׻ɸ֤
          Element::Cur2fix2tar(void) (inline)
                target_time, fixed_time  current_time ˰פ

        ߻ (׻Ѥ߻), ॹƥå,  current_time, 
        λ뤿Υؿޤ.

        Element η׻,  (1), (2) 򷫤֤ޤ.
            (1) current_time  current_time + time_step ޤǷ׻.
            (2) current_time, time_step 򹹿.
        (2) Ǥ, current_time + time_step  Terminal_time ۤʤ
        椷ޤ.

        ɸŪ Element Ǥ,
            owner  current_time + time_step ۤη׻¹Ԥ
            ɬפϤʤ.
        ȹͤ, 嵭Τ褦 Terminal_time ޤ.

        Terminal_time ϲ۴ؿʤΤ, 饹ѹǤޤ.
        㤨, owner ¸ߤʤǥǥǤ Terminal_time ׻Ǥ
        ʤΤǤδؿѹɬפޤ.

        Cur2fix2tar  target_time, fixed_time  current_time
        ˰פؿǤǴȿ׻Ԥʤǥǥ
        ǤϡSet_initial_state  current_time ꤵ줿
        ˤδؿƤɬפޤ

        ACCIDENT:
            owner ¸ߤʤǥ֥Ȥ Element::Terminal_time
              Ѥ.
        </japanese> */

protected:
  // added by Y.Ichikawa (Fri Nov 10 15:02:48 1995)
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Renew_current_time(void)
    {
        current_time += time_step;
        return YES;
    }
    //; ver. 1.3.5 ޤǤ Renew_current_time  Sub_system 
    //; ƤǤ Element ˹ߤ

public:
  // added by Y.Ichikawa (Fri Nov 10 15:02:48 1995)
  //////////////////////////////////////////////////////////////////////
  virtual void Rewind_current_time(void);
  //////////////////////////////////////////////////////////////////////
  virtual void Renew_fixed_time(void);
  //////////////////////////////////////////////////////////////////////
  virtual void Initialize_target_time(void) {}
  //////////////////////////////////////////////////////////////////////
  virtual void Renew_target_time(time_t itrs);
  //////////////////////////////////////////////////////////////////////
  virtual time_t Calculate_iteration_step(void);

        /* <japanese>
          Element::Rewind_current_time(void) current_time ᤹
          Element::Renew_fixed_time(void) fixed_time 򹹿
          Element::Initialize_target_time(void) Element ǤϻȤʤ
          Element::Renew_target_time(time_t itrs)
                target_time 򹹿
          Element::Calculate_iteration_step(void)
                ȿ׻ɸ׻

        current_time, fixed_time, target_time ؿ
        Ѱդޤ

        Rewind_current_time(void)  current_time  fixed_time
        ᤷޤ

        Renew_fixed_time(void) fixed_time  current_time ˿ʤ
        ޤ

        Initialize_target_time(void)  Element ǤϻȤޤ
        Sub_system ǺƻȤޤʤ Element 
        ȤȡElement Ǥ빽ǤФƤ⡢塢
        δؿƤɬפ뤫Ǥ路ϡSub_system
         Initialize_target_time 򻲾ȤƲ

        Renew_target_time(time_t)  target_time 򹹿ޤ
        ȿ׻ֳִ֤Ȥޤ

        Calculate_iteration_step(void) ȿ׻ɸ׻
        ޤ

        (*  *)
        ȿ׻ǤǤ Calculate_iteration_step ɬ
        ƲCalculate_iteration_step  owner 
        Calculate_iteration_step ƤФޤowner 
        Calculate_iteration_step ͤ Renew_target_time 
        ȤϤޤ
        ʤϥ顼åФ exit ޤ

        </japanese> */

public:
  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_from(Receive_port* rp);
  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_to(Receive_port* rp);
        /* <japanese>
              Element::Necessary_time_from(Receive_port*) (virtual)
              Element::Necessary_time_to(Recevie_port*) (virtual)
                ǡɬ׻֤

        Element η׻ɬפʥǡΤ, (Receive_port *rp 
        ٤ǡΥǡϿ֤ޤ.

        Necessary_time_from  Necessary_time_to ޤǤΥǡѥå
        ü rp Ƥʤ, Element Ϸ׻򳫻Ϥ뤳
        ȤǤޤ

        rp üҤξ硣

        RETURN:
            time_t    ǡɬ׻֤
        </japanese> */

public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Initial_time(time_t i_time);
        /* <japanese>
          Element::Initial_time(time_t) (virtual) ν

        Element  current_time, time_step  i_time, 0 ˽ YES
        ֤ޤ.

         time_step  0 ʾξ,
            current_time, time_step  i_time, 0 Ѥߤξ, ٹ
              åϸ YES ֤ޤ.
            ʳξ, 顼åϸ NO ֤ޤ.

        ARGUMENT:
            i_time     (sec).
        </japanese> */

//; 5.  ׻μ¹.
protected:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Initial_output(void) = 0;
public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Initial_work(void);
  //////////////////////////////////////////////////////////////////////
  virtual Element::OperationFlag Do_initial_work(void);

        /* <japanese>
          Element::Initial_output(void) (pure virtual,protected)
                ν

        ̤᤿ľˡcurrent_time ǤνϤޤ
        üҤ˻Ƚͤ񤭹ߤޤcurrnet_time ǤνϤ
        Ǥ YES ֤ޤɬפϥǡƤʤ
        ʤɤơcurrent_time νϤ׻ǤʤäϡNO ֤
        ޤ

        ǤˤäƤϡSet_initial_state(void) ǽνϤ򤹤
        ˤԹ礬褤Τޤ󡣤ȤСΤ
        ͿʸˤäơνϤΤ꤫Ѥꤹ
        ʤɤǤϡInitial_output(void) ϲ⤷ʤ褦
        ơSet_initial_state(void) ǽνϤ򤹤褦
        Ƥ⹽ޤ

          Element::Initial_work(void) (virtual)
                ν

          Element::Do_initial_work(void) (virtual)

        ȤޤʤƤʤ:

        (1) Ǥ٤ƤμüҤΥǡǤСȤ
        ǽǤȽǤơInitial_work Ƥӡinitialworkdone_p 
        YES ꤷơE_DONE ֤ޤ

        (2) ǤʤȤҤȤĤμüҤΥǡǤʤ
        ȤϡȤԲǽǤȽǤơE_UNDONE
        ֤ޤ

        ȤǤˤǤ:

        E_ALREADY ֤ޤ

        </japanese> */

protected:
  //////////////////////////////////////////////////////////////////////
  virtual time_t Calculate_time_step(void) = 0;
        /* <japanese>
          Element::Calculate_time_step(void) (pure virtual,protected)
                ॹƥåפη׻

        饹ǥǥ˱ time_step (sec) ׻, ͤȤ
        ֤ؿƲ.

        Ĥޤ, time_step ɤΤ褦ʾﲼ, ɤΤ褦ͤˤ뤫
        ޤ.

        㤨, time_step  3600 (sec) ꤹ,
            time_t Calculate_time_step(void) { return 3600; }
        Ʋ.

        ޤΤ褦, ǡ k ͤˤ time_step ѹ
        ȤǤޤ.
            time_t Calculate_time_step(void) { return (k > 0) ? 360 : 60; }
            time_t Calculate_time_step(void) { return 360 * pow(10, k); }

        Element::time_step , Element::Work ꤵΤǡ
            time_t Calculate_time_step(void) { time_step = 1; return 1; }
        Τ褦, δؿѹʤǤ

        貾ؿʤΤ, 饹ɬƲ.

        RETURN:
            time_t    η׻Υॹƥå. (sec)
        </japanese> */
protected:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Can_you_calculate θȤƤޤܺ٤ϡ
  //  Element  DESCRIPTION 
  // </japanese>
  Boolean Can_you_calculate0(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Can_you_calculate θȤƤޤܺ٤ϡ
  //  Element  DESCRIPTION 
  // </japanese>
  Boolean Can_you_calculate1(void);

public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ׻ϤβԲĤȽ
  //
  // ޤ, current_time  Element::Terminal_time ʾξ, ׻Ѥ
  // ȽǤ NO ֤ޤ.
  //
  // , ƤνͭüҤ Necessary_time_from, Necessary_time_to
  // ֤ΥǡѥåǽǤ뤳Ȥǧޤ.
  // ǽʾ, ׻¹ԲǽȽǤ YES ֤ޤ.
  // Բǽʾ, ׻¹ԲĤȽǤ NO ֤ޤ.
  //
  // ǡԲǽǤֻ򤷤ƤüҤФơ
  // ׻׵ФޤŪˤϡüҤ Creq å
  // ޤ
  // <pre>
  // RETURN:
  //   YES   ׻¹Բǽξ.
  //   NO    ׻¹Բǽξ.
  //            ׻Ѥߤξ.
  // </pre>
  // </japanese>
  virtual Boolean Can_you_calculate(void) = 0;

protected:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // 1 ƥåʬη׻³
  //
  //ƥåʬη׻³Ʋ.
  //
  // Ū Element Ǥ,
  // <pre>
  //  ǽ, ƾ̤ˤ, current_time ˤͤǼƤ.
  //  Ǥ current_time + time_step ˤ, ͤ
  //    ׻.
  //  δؿλ, ̤ current_time + time_step ˤ
  //    ͤ˹Ƥ.
  // <pre>
  // Σ­褦,  (1)  (4) ˵ҤƲ.
  // <pre>
  //   (1) üҤ.
  //   (2) ׻.
  //   (3) üҤؤν.
  //   (4) ̤ι.
  // </pre>
  // (2) Ǥ, ѥ᡼, current_time ˤ, üҤ
  // ϤѤ, current_time + time_step ˤ, ٤
  // ͤ׻, ǥΣƥåʬŪƲ.
  //
  // δؿ current_time, time_step ѹʤǲ.
  // ɬפʤ time_step 򹹤˺٤ֶʬʬ䤷Ʒ׻Ǥޤ,
  // λϺѿѤ, current_time, time_step ѹʤ
  // դƲ.
  //
  // ׻ɬפƤοͤ, ǡ, ڤӽͭüҤ
  // ϤȤƼǤޤ. äưפǤ.
  //
  // ͽ̤׻Ǥ, YES ֤ޤCalculate Ƿ׻
  // ֥ƥåפ׻, η׻֥ƥåʬΥǡǤ
  // ʤɤ, ׻ʤ뤳ȤǤʤä, NO ֤ޤ
  //
  // 貾ؿʤΤ, 饹ɬƲ.
  //
  // <pre>
  //  ACCIDENT:
  //    δؿ current_time, time_step ѹ줿, 
  //      ȯζ줬.
  // </pre>
  // </japanese>
  virtual Boolean Calculate(void) = 0;

protected:
  //////////////////////////////////////////////////////////////////////
  void Set_data_pack_array(void);
        /* <japanese>
          Element::Set_data_pack_array(void) (protected)
                ǡѥå

        ƤνͭüҤ Send_port::Set_data_pack_array Ѥ, 
        ǡѥå򹹿ޤ.

        ˤ, ŤǡѥåϾõ, ǡѥå
        󤬺ޤ.
        </japanese> */
protected:
  //////////////////////////////////////////////////////////////////////
  void Reduce_data_pack_stock(void);
        /* <japanese>
          Element::Reduce_data_pack_stock(void) (protected)
                ץǡѥåξõ

        ƤνͭüҤ Receive_port::Reduce_data_pack_stock Ѥ,
        üҤ˷׻λԤʤޤ.

        ˤ, üҤץǡѥåõ, ǡ
        å󤬹ޤ.
        </japanese> */
protected:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Print_status(void) { return YES; }
public:
  //////////////////////////////////////////////////////////////////////
  Boolean Record_status(void);
        /*
        #   Element::Print_status(void)
        #   Element::Record_status(void)
        */
protected:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // time_step Ƥ, Calculate ¹Ԥ Work
  // </japanese>
  Boolean Work0(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // time_step , Calculate Ƿ׻ Work
  // </japanese>
  Boolean Work1(void);

    // added by Y.Ichikawa (Fri Nov 10 15:55:28 1995)
  //////////////////////////////////////////////////////////////////////;
  Boolean Work2(void); //; ȿ׻ Work 

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ɸб Work 
  // </japanese>
  Boolean Work3(void);

public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Work(void) = 0;
        /* <japanese>
          Element::Work0(void) (protected)
          Element::Work1(void) (protected)
          Element::Work2(void) (protected)
          Element::Work(void) (virtual)
                current_time, Terminal_time ֤η׻

        Calculate, Calculate_time_step ʤɤѤ, Terminal_time ޤǤ
        ׻¹Ԥޤ.

        ޤ, Element::Can_you_calculate ͤ NO ʤ, ׻
        ĤȽǤ, NO ֤ޤ.

        ͤ YES ξ, Element::Can_you_calculate ͤ YES
        Ǥ¤, Element::Calculate, current_time += time_step 
        ֤¹Ԥޤ.

        Ǹ, Element::Set_data_pack_array, Element::Report_to_sender
        ¹Ԥ, YES ֤ޤ.

        RETURN:
            YES   ׻¹Ԥ.
            NO    ׻Բǽξ.
                    ׻Ѥߤξ.
        </japanese> */

public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Do_you_reach_target_time(void);
    // added by Y.Ichikawa
    //; ȿ׻ target_time ãɤȽǤޤ
    //; ãƤ YES, ãƤʤ NO ֤ޤ

    virtual Boolean Do_you_converge(void);
    //; added by Y.Ichikawa
    //; Ǵ֤ȿ׻μ«Ƚꤷޤ
    //; ȿ׻ԤʤǤɬƲ
    //; « YES, «Ƥʤ NO ֤Ʋ
    //; ʤϥ顼åФ exit ޤ

public:
  //////////////////////////////////////////////////////////////////////
  virtual void Flush_data(void);
    //; added by Y.Ichikawa
    //; ȿɸޤǤΥǡνϤȡǡѥåν
    //; ؿǤ

protected:
  //////////////////////////////////////////////////////////////////////;
  virtual void Send_stocked_data(void);
    //; added by Y.Ichikawa
    //; ȿɸޤǤˤޤäǡϤؿǤ
    //; üҤ Send_data(time_t, Data<T>) ѤƲ
    //; ȿ׻ԤʤǤɬƲ
    //; ʤϥ顼åФ exit ޤ

public:
  //////////////////////////////////////////////////////////////////////
  // Register request of calculation to this element
  //
  // <japanese>
  // Ǥ׻뤳ȤˤʤäɬפȤʤǡǤ뤫
  // ؿ Can_you_calculate() ¹Ԥ뤳ȤǡǤ¸
  // ¾Ǥط׻ãޤ
  //
  // ؿ Can_you_calculate() ɤǡǤؤη׻ä
  // ȤϿޤǤ Work ǡθޤ
  //
  // δؿϡǤüҤƤФޤʬϤǤϡδؿ
  // ƤФ뤳ȤϤޤüҤϡ׻׵üҤž
  // ǡüҤνͭԤǤʬϤˤϷ׻׵ʤ
  // 
  //
  // ŬڤǤϡѾ饹ǺƤ
  // </japanese>
  virtual void Creq(void) {
    time_t time_step_backup = time_step;
    time_step = Calculate_time_step();
    Can_you_calculate();
    time_step = time_step_backup;
    mCreqp = YES;
  }

//; 6.  ϵǽ.
protected:
  //////////////////////////////////////////////////////////////////////
  void Print_objects(FILE* fp, char* head, char* title,
                       void** obj_array, int number_of_objs);
public:
  //////////////////////////////////////////////////////////////////////
  void Print_receive_ports(FILE* fp = stderr, char* head = "    ")
        { Print_objects(fp, head, "Receive_ports :",
                        receive_port_array, number_of_receive_ports); }
  //////////////////////////////////////////////////////////////////////
  void Print_send_ports(FILE* fp = stderr, char* head = "    ")
        { Print_objects(fp, head, "Send_ports :",
                        send_port_array, number_of_send_ports); }
  //////////////////////////////////////////////////////////////////////
  void Print_connection(FILE* fp = stderr, char* head = "    ");
        /* <japanese>
          Element::Print_objects(FILE*,char*,char*,void**,int)
                (protected)     ֥ͭȤξ
          Element::Print_receive_ports(FILE*,char*) (inline)
                ͭüҤξ
          Element::Print_send_ports(FILE*,char*) (inline)
                ͭüҤξ
          Element::Print_connection(FILE*,char*)
                ³ν

        </japanese> */

//;  7.  ǽ֤ե¸뵡ǽ
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Ǥκǽ֤եݥ fp ǻꤵե˽񤭹
  // Ȥ򤹤дؿǤ桼ǥǥǤϡδؿ
  // ɬʤФʤޤ
  //
  // ǽ֤񤭹ͤϼΤ褦ˤƤ
  //
  // (data_string)
  //
  // </japanese>
  virtual void Save_terminal_state(FILE* fp) = 0;

  //////////////////////////////////////////////////////////////////////
  virtual void Terminal_state(FILE* fp);

//; 8.  󥹥ȥ饯, ǥȥ饯, ¾δŪǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a pointer to a new Element
  virtual Element* NewElement() { return NULL; }

  //////////////////////////////////////////////////////////////////////
  // Initilization with the information which is given through a file.
  //
  // <japanese>
  // ǤΥ饹̾ե뤫ɤߤȤäƤĤꤢʬϥǥ
  // Ѥ륳󥹥ȥ饯ؿ̵Υ󥹥ȥ饯Ǻ
  // Ȥ˸ƤФ뤳ȤꤷƤޤ
  // 㤨Сclassname, objname, objnum ե뤫ɤ߼Ƥ
  // ơ
  //
  // <pre>
  //    Element* p = new_element(classname);
  //    p->Init(objname, objnum, fp);
  // </pre>
  //
  // Τ褦ˤơǤƤޤ
  //
  // ǤΥ饹̾ե뤫ɤߤȤäƤĤꤢʬϥǥ
  // Ѥ褦ˤ뤿ˤϡδؿƤɬפ
  // 
  // Init(objname, objnum, fp) ̤ιǤƤФ줿Ȥ˥顼
  // åɽߤ褦ˤƤޤ
  // </japanese>
  //
  virtual Boolean Init(char*, int, FILE*)
  { cerr << "Init((char*, int, FILE*) is not defined."
         << endl; exit(1); }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* o_name, int o_num,
                 int num_of_objs = 0, char* p_str = NULL)
        { return ((Object::Init(o_name, o_num) == YES)
                    && (Set_object_array(num_of_objs) == YES)
                    && (Set_data_string(p_str) == YES)) ? YES : NO; }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* o_name, int o_num, char* p_str)
        { return ((Object::Init(o_name, o_num) == YES)
                    && (Set_object_array(0) == YES)
                    && (Set_data_string(p_str) == YES)) ? YES : NO; }
    // modified by Y.Ichikawa (Mon Jan  8 13:17:42 1996)

  //////////////////////////////////////////////////////////////////////
  Element(char* o_name, int o_num,
            int num_of_objs = 0, char* p_str = NULL);

  //////////////////////////////////////////////////////////////////////
  Element(char* o_name, int o_num, char* p_str);

  //////////////////////////////////////////////////////////////////////
  Element(void);
        /* <japanese>
          Element::Init(char*,int,int,char*)      ؿ
          Element::Init(char*,int,char*)          ؿ
          Element::Element(char*,int,int,char*)   󥹥ȥ饯
          Element::Element(char*,int,char*)       󥹥ȥ饯
          Element::~Element(void) (virtual)       ǥȥ饯

        쥯饹 Object Υ󥹥ȥ饯 o_name, o_num, p_string Ϥ
        ޤ.
        Object::object_name ˤ o_name ԡ, Object::object_number
         o_num (ǥե 0) ꤵޤ.

        ˥ѥ᡼ʸ p_string Ϥ, data_string  p_string
        򥳥ԡǤޤ. p_string , Element::Set_parameter_string 
        ޤ.

        㤨, Τ褦ѤƲ.
            Element a("abc", 123, "1.2 345 6.78");

        ǥեȥ󥹥ȥ饯 class_name, object_name, object_number
         "Element", NULL, -1 ꤷޤ.
        ξ Object::Init ѤƥФƲ.

        ARGUMENT:
            o_name    ֥̾.
            o_num     ֹ֥. ǥե 0.
            p_string  ѥ᡼ʸؤΥݥ.
            p_string  ѥ᡼ʸؤΥݥ.
        </japanese> */
  //////////////////////////////////////////////////////////////////////
  ~Element(void);
        /* <japanese>
          Element::~Element  (virtual)    ǥȥ饯

        object_list, receive_port_array, send_port_array  delete ޤ.

        object_list  Construct ¹Ը, delete  NULL ꤵޤ.
        NULL ݥ󥿤 delete Ƥ̵Ǥ뤳ȤݾڤƤޤ.
        (.ȥ饦ȥå : ץߥ󥰸áܡ 裲, ƣ
        , ȥåѥ, pp.653, 1993. )
        </japanese> */

};

#endif  // ! element_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
