/*;  jointbl.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl jointbl.h\quad\thepage}} */
/*;  $$Date: 2000/09/03 13:51:49 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/jointbl.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) jointbl.h -- header file for class JointBl

/*
{\ {\setlength{\parskip}{4pt}% latex-mode
...
}% end of latex-mode }
*/

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "common.h"        //; ̴Ķ
#include "list.h"          //; 饹 List 
#include "element.h"       //; 饹 Element 
#include "p_tmp.h"         //; ƥץ졼ȥ饹 Receive, Send 

/*----------------------------------------------------------------------
CLASS
    JointBl

    Joint multiple inputs and output the sum of them

DESCRIPTION
    An object of class JointBl has a number of receive ports which
    receives Data<double> data and send sum of received data at constant
    time intervals through a send port.

    <japanese>
    class JointBl Υ֥ȤϡʣĿ receive port 
     Data<double> ΥǡȤꡢޤǡȲᤷơ
    ֳִ֤ǡ¤ơsend port Фޤ
    </japanese>

    <japanese>
    [ǥ̾]JointBl

    [ǽ]class JointBl Υ֥ȤϡʣĿ receive port 
     Data<double> ΥǡȤꡢޤǡ
    ᤷơֳִ֤ǡ¤ơsend port 
    ޤɽ

         a[0]r[0](t+Dt) + a[1]r[1](t+Dt) + ... + a[n-1]r[n-1](t+Dt)

    ׻ƽϤޤn: receive port θĿa[0], a[1],
    ..., a[n-1]: Dt: ֳִ֡k: Ǥ, r[i](t): iܤμü
    Ҥλ t μ͡

    [üҤλ]üҤθĿϡǤդǤ

    üҤθĿϡ¤եǤϡInit Τղþʬ
     Init(char* o_name, int o_num, int numinput)
    ǻꤷޤޤϡдؿ Set_ports(int n) ǻꤷޤ
    üҤ갷ǡѥåϡData<double> ǤüҤȤˡǡ
    Ͽ郎äƤƤ⹽ޤ󡣼üҤΥ֥̾ϡ"rp"
    ֹ֥ϡ0, 1, ..., n-1 Ǥ

    [üҤλ]Data&ld double&gt ǡФ롢֥
    ̾"sp"ֹ֥ 0 üҤ1Ļޤ

    [ѥ᥿λ]ѥ᥿ϡüҤǡϤֳִ
    Dt (sec), ƼüҤؤΨ a[0], a[1], ... a[n-1] Ǥ

    [֤λ]̤ϡcurrent_time Ǥ
    ֤ꤹؿȤơ
    <pre>
	void Set_initial_state(time_t ct);
    </pre>
    ѰդƤޤϡcurrent_time  ct ꤷޤ

    []üҤϡŪѲǡϤΤȤ
    ǡޤդͿΤȤơĺλȤλ
    ǤͤǡѥåˤޤȤơüҤͿޤJointBl 
    ϡġμüҤ顢mDt Υǡơ줾ͤ
    褸¤ȤꡢüҤФޤ

    </japanese>

----------------------------------------------------------------------*/

class JointBl : public Element {
  int mDt;    // sampling interval
  double* mA; // multipliers

  //////////////////////////////////////////////////////////////////////
  // Receive ports
  Receive<double>* mRps;

  //////////////////////////////////////////////////////////////////////
  // The number of receive ports
  int mNumRps;

  //////////////////////////////////////////////////////////////////////
  // Send port
  Send<double> mSp;

  //////////////////////////////////////////////////////////////////////
  void Register_receive_ports(void);

  //////////////////////////////////////////////////////////////////////
  void Register_send_ports(void) { Register((void*)&mSp); }

private:
  // To prohibit substitution.  No implementation
  JointBl& operator=(const JointBl& rhs);

public:
  //////////////////////////////////////////////////////////////////////
  // Returns a pointer to a new JointBl
  JointBl* NewElement() { return new JointBl(); }

  //////////////////////////////////////////////////////////////////////
  // Init function for SCF.
  //
  // <japanese>
  // ¤ե init ؿ"Init Τɲþ" ʬˤϡ
  // üҤθĿ񤭤ޤ
  // </japanese>
  // <pre>
  // JointBl 3
  // </pre>
  Boolean Init(char* o_name, int o_num, FILE* fp);

  //////////////////////////////////////////////////////////////////////
  // [in] o_name   - object name
  // [in] o_num    - object number
  // [in] numinput - number of receive ports
  Boolean Init(char* o_name, int o_num, int numinput);

  //////////////////////////////////////////////////////////////////////
  // Constructor
  JointBl(char* o_name, int o_num, int numinput)
    : Element(),
    mRps(NULL), mNumRps(0), mSp("sp", 0)
	{
      class_name = "JointBl";
      Init(o_name, o_num, numinput);
	}

  //////////////////////////////////////////////////////////////////////
  // Default constructor
  JointBl(void)
    : Element(),
    mRps(NULL), mNumRps(0), mSp("sp", 0)
	{ class_name = "JointBl"; }

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~JointBl(void)
	{
      delete [] mRps;
      delete [] mA;
	}

//; ѥ᡼ꡦ̤ν
public:
  //////////////////////////////////////////////////////////////////////
  // Read in paramters
  //
  // <pre>
  // void Set_parameter(FILE* fp) {
  //  fscanf(fp, "%d", &mDt);
  //  for (int i; i < mNumRps; i++) fscanf(fp, "%lf", &(mA[i]));
  // }
  // </pre>
  void Set_parameter(FILE* fp) {
    fscanf(fp, "%d", &mDt);
    for (int i; i < mNumRps; i++) fscanf(fp, "%lf", &(mA[i]));
  }

  //////////////////////////////////////////////////////////////////////
  // Set the number of receive ports
  void Set_port(int n);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // current_time  fscanf(fp, "%ld", &current_time) ɤ
  // ȤäͤѹޤɤߤȤäͤʤѹޤ
  // </japanese>
  //
  // <pre>
  // void Set_initial_state(FILE* fp) {
  //  time_t tnow; fscanf(fp, "%ld", &tnow);
  //  if (tnow > 0) current_time = tnow;
  // }
  // </pre>
  //
  void Set_initial_state(FILE* fp) {
    time_t tnow; fscanf(fp, "%ld", &tnow);
    if (tnow > 0) current_time = tnow;
  }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // current_time  ct ѹޤ
  // ct ʤѹޤ
  // </japanese>
  void Set_initial_state(time_t ct) {
    if (ct > 0) current_time = ct;
  }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // current_time  fprintf(fp, "(%ld)", current_time) ǽ񤭽Фޤ
  //
  // </japanese>
  void Save_terminal_state(FILE* fp) {
    fprintf(fp, "(%ld)", current_time);
  }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // current_time  ct ˳ǼäƤޤ
  // </japanese>
  void Save_terminal_state(time_t& ct) {
    ct = current_time;
  }

//; ׻ǽ
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ϤûơüҤ˽񤭽Фޤ
  // </japanese>
  Boolean Initial_output(void);

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // time_step ϡmDt ˸Ǥ
  // </japanese>
  time_t Calculate_time_step(void) { return mDt; }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ٤ƤμüҤǡNecessary_time_from, Necessary_time_to 
  // ֤δ֤ΥǡǤз׻ǽȽǤޤ
  // </japanese>
  Boolean Can_you_calculate(void) { return Can_you_calculate0(); }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ׻ֳִ֤Ƥ calculate ¹Ԥ Work0 ѤƤ
  // 
  // </japanese>
  Boolean Work(void) { return Work0(); }

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // <pre>
  // a[0]r[0](t+Dt) + a[1]r[1](t+Dt) + ... + a[n-1]r[n-1](t+Dt)
  // </pre>
  // üҤޤ
  // </japanese>
  Boolean Calculate(void);

};

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
