/* output_f.h {\documentstyle{jsrc2latex}\rhead{\sl output\_f.h\quad\thepage}
   \font\tenrm=cmtt10}Receive prot model for output files
{\ \begin{verbatim}
$Revision: 1.6 $ $Date: 1996/08/24 10:18:01 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode
}% end of latex-mode }

This header file defines a receive port model class "Output_file" for
output files.

The class hierachy relating class "Output_file" is as follows:

        Object <-- Port <--+
                           |
     +---------------------+
     |
     +--- Receive_port <-- Output_port <-- Output_file  <--+-- O_file<T>
                                                           |
                                           Port_type<T> <--+
*/

#if !defined(output_f_h)
#define output_f_h

#include <stdio.h>
#include <time.h>
#include <limits.h>

#include "output.h"		// class Output_port

//	Definition of class Output_file

/* ---------------------------------------------------------------------
CLASS
    Output_file

DESCRIPTION

	Class Output_file stores "output file name" in "Object::object_name".
	Functions to output received datapacks to the output file are
	defined as its member functions.

HIERARCHY

    EXAMPLE

        Object <-- Port <--+
                           |
     +---------------------+
     |
     +--- Receive_port <-- Output_port <-- Output_file  <--+-- O_file<T>
                                                           |
                                           Port_type<T> <--+

    END

--------------------------------------------------------------------- */

class Output_file : public Output_port {
	/*
	##	Output_file		class for creating a receive port associated to
	                    a file for output

	Class Output_file stores "output file name" in "Object::object_name".
	Functions to output received datapacks to the output file are
	defined as its member functions.
	*/

//	0.	ǡ, ؿ.
protected:
  //////////////////////////////////////////////////////////////////////
  int line_length;		// ϥǡʸκĹ
  //////////////////////////////////////////////////////////////////////
  time_t last_data_time;	// ǽϥǡΥǡϿ
  //////////////////////////////////////////////////////////////////////
  char* comment_string;   // ե˽񤭽Фʸ

public:
  //////////////////////////////////////////////////////////////////////
  int Line_length(void) { return line_length; }
  //////////////////////////////////////////////////////////////////////
  time_t Last_data_time(void) { return last_data_time; }
		/* <japanese>
			Output_file::Line_length(void) (inline)
				line_length ֤
			Output_file::Last_data_time(void) (inline)
				last_data_time ֤
        </japanese>
		*/

//	1.	³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  virtual char* Data_pack_type(void) = NULL;
		/* <japanese>
			Output_file::Data_pack_type(void) (pure virtual)
				ǡݥѥåɽʸ֤

		Receive_port ν貾۴ؿǤ, Output_file Ǥ貾۴ؿ
		Ƥޤ. Output_file 饹, ǡѥåɽ
		ʸؤΥݥ󥿤֤褦ɬƲ.

		㤨, ǡѥå "INT" Ǥ饹Ǥ,
			char* Data_pack_type(void) { return "INT"; }
		Τ褦ޤ.

		³, 󥯻ˤ, üҤ Data_pack_type , üҤ
		Data_pack_type Ȥפ뤳Ȥǧޤ.

		RETURN:
			char*		ǡѥåɽʸؤΥݥ.
        </japanese>
		*/

//	2.	ϥեν񤭹ߵǽ.
public:
  //////////////////////////////////////////////////////////////////////
  //  copy the string "cmt" to the data member "comment_string".
  //  The string copied to "comment_string" will be printed out into
  //  the output file, while the member function "Write_hedda is
  //  executed. See the source code of "Write_hedda".
  void Set_comment_string(char* cmt);

protected:
  //////////////////////////////////////////////////////////////////////
  // Writes a header part to output file. The header contains
  //
  // <ul>
  //  <li> file name
  //  <li> information about the send port which sends data to this port
  //  <li> comment_string if the comment_string of the send port is not
  //       null
  // </ul>
  //
  // If the output file cannot be opened, this function returns NO.
  virtual Boolean Write_hedda(void);
		/* <japanese>
			Output_file::Write_hedda(void)    ϥǡν񤭹

		ϥեΥإåȤ, ϿǡѥåƤü
		Ҥ˴ؤ, ϰϵޤ.

		إåʬˤ, Ƭ comment (ǥե '#') 񤭹, 
		ȥȤΤ, νϥեƤϥեȤ
		뤳ȤǤޤ.

		ϥե뤬ץǤʤƤ⶯λޤ.

		إå񼰤ѹ, 饹ǺƲ.

		RETURN:
			YES	˵Ǥ.
			NO	ϥե뤬ץǤʤä.
        </japanese>
		*/

protected:
  //////////////////////////////////////////////////////////////////////
  // Writes the content of datapack <b>d_pack</b> into the area to
  // which <b>str</b> points. The space which <b>str</b> points to must
  // be large enough to store the content of the datapack.
  //
  // Since this function is a pure virtual function, implementation of
  // this function is compulsory in derived classes.
  virtual void Make_string_data(char* str, void* d_pack) = NULL;
		/* <japanese>
			Output_file::Make_string_data(char*,void*) (pure virtual)
				ϥǡʬκ

		ϥե룱Ԥˤ, ĤΥǡѥåΥǡƤ򵭽Ҥޤ.

		ĤΥǡѥå d_pack ΥǡƤϥե룱ʬʸ
		ǡ str ؿ饹Ʋ.

		str εΰ, δؿ¦ Input_file::Write_output_datas
		¦ǳݤޤ.

		貾ؿʤΤ, 饹ɬƲ.

		ARGUMENT:
			str		ϥǡƤ񤭹Υɥ쥹.
			d_pack	str Ѵǡѥå.
        </japanese>
		*/

public:
  //////////////////////////////////////////////////////////////////////
  // Outputs data.
  virtual void Output_data(void);
		/* <japanese>
			Output_file::Output_data(void)	ϥǡν񤭹

		ޤ, Ĺ line_length κʸҡפ˳ݤ, ݤǤ
		ϥ٥룵Ƕλ.
		ޤ, ϥե object_name ⡼ "a" ǥץ, 
		Ǥʤ return .

		, ǡѥå data_pack_array Ƭ˳ǼƤ
		ǡѥå֤ Output_file::Make_string_data Ѥƥǡ
		ʸ, ϥե˵.

		() ǡѥå, ΥдؿǼޤ
		ǥ Output_file , üҤǥǤä, ǡѥå
		򥹥ȥå뵡ǽϤޤ

		block  1 ʾξ, ϥǡ block Խ񤭹ऴȤ "\n"
		񤭹, ڤĤ. block Υǥեͤ -1. (ǥե
		ȤǤ϶ڤʤ. )

		ARGUMENT:
			block		ϥǡζڤԿ. 0 ʲʤڤʤ.
						ǥե -1.
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  time_t Necessary_time_from(void) { return last_data_time; }
  //////////////////////////////////////////////////////////////////////
  time_t Necessary_time_to(void) { return LONG_MAX; }
		/* <japanese>
			Port::Necessary_time_from(void) (virtual)
			Port::Necessary_time_to(void) (virtual)
				ǡɬ׻֤

		쥯饹 Receive_port β۴ؿǤ.
		üҤ, Necessary_time_from ʹ Necessary_time_to 
		Υǡѥåõʤ褦˻ؼ뤿ޤ.

		۴ؿʤΤ饹ǺǤޤ.

		RETURN:
			time_t	ǡɬ׻ (sec).
        </japanese>
		*/

//	3.	ɽǽ.
  //////////////////////////////////////////////////////////////////////
  void Cprint(FILE* fp_out, char* head);
  //////////////////////////////////////////////////////////////////////
  void Oprint(FILE* fp_out = stderr, char* head = "    ");
		/* <japanese>
			Output_file::Cprint(FILE*,char*) (virtual)
				֥Ⱦν (owner ξޤ)
			Output_file::Oprint(FILE*,char*) (virtual)
				֥Ⱦν (owner ξޤ)

		ϥե̾ȤüҤμüҤξե fp_out
		˽Ϥޤ.

		ؿ Cprint ϥ饹 Object β۴ؿʤΤ, 饹Ȥѹ
		Ǥޤ.

		Object::Cprint ¿Ƥ뤿, fp_out Ϥʤ
		 stderr ˽Ϥ, λ, head Ϥʤ head 
		ꤵޤ.

		ARGUMENT:
			fp_out	֥ȾϤեؤΥݥ.
		    			Ϥʤ stderr Ȥʤ.
			head		ʸؤΥݥ.
						fp_out ϤʤΥǥեͤϣ.
        </japanese>
		*/

//	4.	ǽ, 󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* f_name, int l_len = 128);
		/* <japanese>
			Output_file::Init(char*,int)	ؿ

		ޤ, Object::Init Ѥ f_name  object_name ˥ԡޤ.
		object_number  0 ꤷޤ. λ, ͤ NO ʤ NO
		֤ޤ.

		 line_length  0 礭, ȽǤ NO 
		ޤ. line_length  0 ʲʤ, line_length  l_len 
		 YES ֤ޤ. l_len Υǥեͤ, 128 Ǥ.

		ARGUMENT:
			f_name	ϥե̾.
			l_len		ϥǡʸκĹ. ǥե 128.
		RETURN:
			YES		˽Ǥ.
			NO		ꤵ줿.
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  Output_file(char* f_name, int l_len = 128);
  //////////////////////////////////////////////////////////////////////
  Output_file(void);
  //////////////////////////////////////////////////////////////////////
  ~Output_file(void);
		/* <japanese>
			Output_file::Output_file(char*,int)			󥹥ȥ饯
			Output_file::Output_file(void)				󥹥ȥ饯
			~Output_file::Output_file(void) (virtual)	ǥȥ饯

		Output_file(char*,int) , 쥯饹 Output_port Υǥեȥ
		󥹥ȥ饯Ѥ, Output_file::Init Ѥ, Фʲ
		ޤ.
			class_name		: "Output_file"
			object_name		: f_name (ҡפ˥ԡ)
			object_number		: 0
			owner				: NULL
			sender			: NULL
			line_length		: l_len (ǥե 128)
			last_data_time	: LONG_MIN

		ǥեȥ󥹥ȥ饯, Output_file::Init ¹Ԥޤ.
		η, ʲȤʤޤ.
			object_name		: NULL
			object_number		: -1
			line_length		: -1
		¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
		ξ, Output_file::Init ˤФƲ.

		ǥȥ饯, ܴ쥯饹 Object  virtual Ƥޤ.

		ARGUMENT:
			p_name	ϥե̾.
			l_len		ϥǡʸκĹ. ǥե 128.
		EXIT:
			5			object_name εΰݤǤʤ.
						(Object::Init)
        </japanese>
		*/

};


#endif	//	! output_f_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.6 $

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */
