/* p_tmp.h {\documentstyle{jsrc2latex}\rhead{\sl p\_tmp.h\quad\thepage}
   \font\tenrm=cmtt10}Port models for template datapack
{\ \begin{verbatim}
$Revision: 1.21 $ $Date: 1996/08/15 08:19:29 $  by KyotoHydro
\end{verbatim}}

This header file declares the port classes Send<T>, Receive<T>,
ReceiveNointp<T>, Relay<T>, I_file<T>, and O_file<T>.

The class Send<T> is the class of ports for sending datapacks of type
Data<T>.

The model Receive<T> and ReceiveNointp<T> are classes of ports for
receiving datapacks of type Data<T>. The port of class ReceiveNointp<T>
does not provide those functions to interpolate data values between data
times. The port of class Receive<T> provides those functions to
interpolate between data times.

The model Relay<T> is the model for relaying datapacks of type Data<T>.

The model I_file<T> is the model for reading datapacks of type Data<T>
from a file.

The model O_file<T> is the model for writing datapacks of type Data<T>
into a file.

*/

#if !defined(p_tmp_h)
#define p_tmp_h

#include <stdio.h>
#include <time.h>       // definition of types time_t, and tm

#include "datapack.h"   // definition of class Data_pack
#include "receive.h"    // definition of class Receive_port
#include "send.h"       // definition of class Send_port
#include "relay.h"      // definition of class Relay_port
#include "input_f.h"    // definition of class Input_file
#include "output_f.h"   // definition of class Output_file

#include "data2str.h"   // prototype declaration of functions for
                        // transforming data strings

/*----------------------------------------------------------------------
CLASS
    Port_type

    Template class for creating a class which gives data_pack_type

KEYWORDS

    keywords: data_pack_type, Dataconnectiontype, Brokenline, Pulse, Step

DESCRIPTION
    Template class Port_type&lt T&gt has character string
    "data_pack_type" as a data member in which the name of datapack type
    will be stored.

    <japanese>
    ƥץ졼ȥ饹 Port_type&lt T&gt ϡdatapack η̾Ǽ
    data_pack_type ǡФȤ륯饹Ǥ

    ü (Port) η򤽤üҤꤢĤ datapack η̾ɽޤ

    </japanese>

    Template classes Send&lt T&gt, Receive&lt T&gt, ReceiveNointp&lt
    T&gt, Relay&lt T&gt, I_file&lt T&gt, O_file&lt T&gt will be derived
    using multi-inheritance from this class Port_type&lt T&gt and
    Receive_port, Send_port, Relay_port, Input_file, Output_file,
    respectively.  For example, Send&lt T&gt will be derived from
    Send&lt T&gt and Send_port.

    <japanese>
    ƥץ졼饹 Send&lt T&gt, Receive&lt T&gt, ReceiveNointp&lt
    T&gt, Relay&lt T&gt, I_file&lt T&gt, O_file&lt T&gt
    줾졢
    Receive_port, Send_port, Relay_port, Input_file, Output_file 
    Υƥץ졼ȥ饹 Port_type&lt T&gt Ȥ¿ŷѾƺ
    ޤ

    㤨СSend&lt T&gt ϡSend_port Port_type&lt
    T&gt Ȥ¿ŷѾƺޤ

    </japanese>

    EXAMPLE
    Object <-- Port <-- Sender_port <-- Send_port <--+-- Send<T>
                                                     |
                                     Port_type<T> <--+
    END

    In Receive&lt T&gt, Send&lt T&gt, Relay&lt T&gt, ..., the pure
    virtual function Data_pack_type(void) declared in class Port will be
    implemented so it returns Port_type&lt T&gt::data_pack_type.

    This implementation of Data_pack_type makes certain that all the
    ports Receive&lt T&gt, Send&lt T&gt, ... which treat the same data
    type Data&lt T&gt returns the same string.

    <japanese>
    ºݤ˻Ȥ뤳ȤˤʤŪüҷ Port_type&lt double&gt,
    Port_type&lt int&gt, Port_type&lt long&gt, Port_type&lt
    OHymosString&gt ϡp_itmp.cc ǡinstantiate ޤʤ
    p_itmp.cc 

    EXAMPLE
      template class Port_type<double>;
      template class Port_type<float>;
      template class Port_type<int>;
      template class Port_type<long>;
      template class Port_type<OHyMosString>;
      char* Port_type<double>::data_pack_type = "double";
      char* Port_type<float>::data_pack_type = "float";
      char* Port_type<int>::data_pack_type = "int";
      char* Port_type<long>::data_pack_type = "long";
      char* Port_type<OHyMosString>::data_pack_type = "OHyMosString";
    END

    ȵҤƤޤ
    </japanese>

----------------------------------------------------------------------*/

//  Definition of template class Port_type<T>

template <class T>
class Port_type {
    /*
    # port_type template class for creating a class which gives datapack
                type

    Templace class Port_type<T> has character string "data_pack_type" as
    a data member in which the name of datapack type will be stored.

    Template classes Send<T>, Receive<T>, ReceiveNointp<T>, Relay<T>,
    I_file<T>, O_file<T> will be derived using multi-inheritance from
    this class Port_type<T> and Receive_port, Send_port, Relay_port,
    Input_file, Output_file, respectively.  For example, Send<T> will be
    derived from Send<T> and Sendq_port.

    In Receive<T>, Send<T>, Relay<T>, ..., the pure virtual function
    Data_pack_type(void) declared in class Port will be implemented so
    it returns Port_type<T>::data_pack_type.

    This implementation of Data_pack_type makes certain that all the
    ports Receive<T>, Send<T>, ... which treat the same data type
    Data<T> returns the same string.

    */

//  0.  data member
protected:
  //////////////////////////////////////////////////////////////////////
  // Data member.  Name of the datapack type
  static char* data_pack_type;

//  1.  constructor and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Constructor
  Port_type(void) {}

  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~Port_type(void) {}

};

//  definition of template class Data

/*----------------------------------------------------------------------

CLASS
    Data

    Template class of Datapack.  Derived from Data_pack and
    Port_type&lt T&gt.

KEYWORDS

    value, data_pack, data_pack_type, data_time

DESCRIPTION

   <japanese>
   Data&lt T&gt ϡ T Υǡѥå饹Ǥ
   </japanese>

   Data&lt T&gt is a class for treating datapack of type T.
   Data members of class Data&lt T&gt are

    <ul>
      <li> char* data_pack_type (inherited from class Port_type)
      <li> time_t data_time (inherited from class Data_pack)
      <li> T value
    </ul>

    <japanese>
    ºݤ˻Ȥ뤳ȤˤʤŪʥǡ Data&lt double&gt, Data&lt
    int&gt, Data&lt long&gt, Data&lt OHymosString&gt ϡp_itmp.cc ǡ
    instantiate ޤʤp_itmp.cc 

    EXAMPLE
      template class Data<double>;
      template class Data<float>;
      template class Data<int>;
      template class Data<long>;
      template class Data<OHyMosString>;
    END

    ȵҤƤޤ

    libohymos.a Ȥˡp_itmp.cc ϥѥ뤵졢
    object module  libohymos.a ˼Ͽޤ

    ΤᡢData&lt double&gt, Data&lt float&gt, Data&lt int&gt,
    Data&lt long&gt, Data&lt OHyMosString&gtϡOHyMOS Ȥ߹ߤη
    ƻѤǤޤ

    </japanese>

-----------------------------------------------------------------------*/

template <class T>
class Data : public Data_pack, public Port_type<T> {
    /*
    # Data  template class of Datapack

    Data<T> stores
        data value: value
    and defines
        function to set the data value
        function to retrieve the data value
    */

//  0.  data member
protected:
  //////////////////////////////////////////////////////////////////////
  // Data member. <b>value</b> stores a data value of Type T.
  //
  T value;  // data value

//  1.  set the data value and retrieve the data value
public:
  //////////////////////////////////////////////////////////////////////
  // set the data time and the data value
  //
  // [in] t -- data time
  // [in] v -- value
  //
  void Set_data(time_t t, const T& v) { Set_data_time(t); value = v; }

  //////////////////////////////////////////////////////////////////////
  // Get data time and data value
  //
  // [out] t -- data time
  // [out] v -- data value
  void Get_data(time_t& t, T& v) { t = data_time; v = value; }

// added by ichikawa.
// 1994.12.9.
// ---<from here>-----------------------------------------------

public:
  //////////////////////////////////////////////////////////////////////
  // Returns data value
  T Value(void) { return value; }

// ---<to here>-------------------------------------------------

//  2.  constructor and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Constructor
  //
  // The constructor uses the constructors of the base classes.
  // The data members are initialized by those base classes'
  // constructors as follows:
  //
  // <pre>
  //        data_time     : 0
  //        value         : 0
  // </pre>
  Data(void) : Data_pack(), Port_type<T>() {}

  //////////////////////////////////////////////////////////////////////
  // Destructor
  //
  // The destructor is delared as virtual in the base class Data_pack.
  ~Data(void) {}

//  3.  transformation between data and string
public:
  //////////////////////////////////////////////////////////////////////
  // data string "data_str" is transformed into datapack
  Data(char*& data_str);

  //////////////////////////////////////////////////////////////////////
  // datapack will be printed into a file in simple data format
  void Dpack2file(FILE* fp);

  //////////////////////////////////////////////////////////////////////
  // datapack will be printed into a file in simple data format
  void Dpack2ofile(FILE* fp);

  //////////////////////////////////////////////////////////////////////
  // datapack value will be set to be a value which is read in from
  // data string.
  void Dstr2dpack(char*& data_str);

};

// definition of teplate class ReceiveNointp

/*----------------------------------------------------------------------
CLASS
    ReceiveNointp

    ReceiveNointp&lt T&gt is a class of ports which receive
    Data&lt T&gt

DESCRIPTION

    ReceiveNointp&lt T&gt receives data of class Data&lt T&gt.
    ReceiveNointp&lt T&gt can be connected with Send&lt T&gt, Relay&lt
    T&gt, and I_file&lt T&gt.

    The port of class ReceiveNointp&lt T&gt does not provide those
    functions to interpolate data values between data times.  If you
    need functions for interpolation, please look Receive&lt T&gt.

----------------------------------------------------------------------*/

template <class T>
class ReceiveNointp : public Receive_port, public Port_type<T> {

//  connection
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data_pack_type which expresses the type of data to be
  // receided.
  // The user must not change the contents
  // of the area to which data_pack_type points.
  //
  // Data_pack_type() is a pure virtual function declared in the base
  // class Port.  It is implemented here.  It returns data_pack_type
  // of base class Port_type&lt T&gt.
  //
  // <pre>
  //      RETURN:
  //       char*  the pointer to the string which expresses the
  //              type of data to be received.
  // </pre>
  char* Data_pack_type(void) { return data_pack_type; }

//  Get data service function

public:
  //////////////////////////////////////////////////////////////////////
  // If the datapack at time t exist, returns YES and the data
  // value is stored in "v".  Otherwise, returns NO.
  virtual Boolean
  Get_pulse_data(time_t t, T& v);

//  initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new ReceiveNointp&lt T&gt Port
  virtual ReceiveNointp<T>* NewPort() { return new ReceiveNointp<T>(); }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
        { return Receive_port::Init(p_name, p_num); }
  //////////////////////////////////////////////////////////////////////
  ReceiveNointp(char* p_name, int p_num)
        :   Receive_port(p_name, p_num), Port_type<T>()
        { class_name = "ReceiveNointp"; }
  //////////////////////////////////////////////////////////////////////
  ReceiveNointp(void)
        :   Receive_port(), Port_type<T>()
        { class_name = "ReceiveNointp"; }
  //////////////////////////////////////////////////////////////////////
  ~ReceiveNointp(void) {}

};

// Definition of Receive<T>

/*----------------------------------------------------------------------
CLASS
    Receive

DESCRIPTION
    Class Receive&lt T&gt receive Data&lt T&gt, a sequence of a pair of
    time and data of type T.

    The data connection type of these sequence of data can be selected
    from

    <dl>
      <dt> Brokenline
      <dd> The data are connected with broken line.

      <dt> Step
      <dd> The value is taken constant and equal to the current
           data value from the previous data time till the
           current time.

      <dt> Pulse
      <dd> The value between data times is taken to be zero.
    </dl>

    Receive&lt T&gt can be connected with Send&lt T&gt, Relay&lt T&gt,
    and I_file&lt T&gt.

-----------------------------------------------------------------------*/

template <class T>
class Receive : public Receive_port, public Port_type<T> {
    /*
    # Receive<T>

    Class Receive<T> receive Data<T>, a sequence of a pair of time and
    data of type T.

    The data connection type of these sequence of data can be selected
    from

      Brokenline  The data are connected with broken line.
      Step        The value is taken constant and equal to the current
                     data value from the previous data time till the
                     current time.
      Pulse       The value between data times is taken to be zero.

    Receive<T> can be connected with Send<T>, Relay<T>, and I_file<T>.

    */

// enum declaration
public:
  //////////////////////////////////////////////////////////////////////
  enum DataConnectionType { Brokenline, Step, Pulse };

//  Data member
protected:
  //////////////////////////////////////////////////////////////////////
  DataConnectionType data_connection_type;

//  Connection
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data_pack_type which expresses the type of data to be
  // received.
  //
  char* Data_pack_type(void) { return data_pack_type; }

//  Interpolate service functions

public:
  //////////////////////////////////////////////////////////////////////
  // Set data connection type
  //
  // [in] dct - data connection type
  //
  // <pre>
  //    case 0: data_connection_type = Brokenline
  //    case 1: data_connection_type = Step
  //    case 2: data_connection_type = Pulse
  // </pre>
  void Set_data_connection_type(int dct);

  //////////////////////////////////////////////////////////////////////
  // Returns the data value at time t.
  virtual T Get_data(time_t t);

  //////////////////////////////////////////////////////////////////////
  // If the datapack at time t exist, returns YES and the data
  // value is stored in "v".  Otherwise, returns NO.
  virtual Boolean
    Get_pulse_data(time_t t, T& v);

  //////////////////////////////////////////////////////////////////////
  // Set the data value at time t to reference v
  virtual void Get_data(time_t t, T& v) { v = Get_data(t); }

  //////////////////////////////////////////////////////////////////////
  // Returns the integrated value from time t1 till time t2.  t1 should
  // be less than or equal to t2.
  //
  // When data_connection_type is Pulse, the value summed from t1 till
  // t2 will be returned. The pulse at time t1 is not contained in the
  // sum, and the pulse at time t2 is contained in the sum.
  virtual T Get_data_sum(time_t t1, time_t t2);

  //////////////////////////////////////////////////////////////////////
  T Get_mean_data(time_t t1, time_t t2)
    { return Get_data_sum(t1, t2) / (t2 - t1); }

//  3.  Initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new Receive&lt T&gt Port
  virtual Receive<T>* NewPort() { return new Receive<T>(); }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
        { return Receive_port::Init(p_name, p_num); }
  //////////////////////////////////////////////////////////////////////
  Receive(char* p_name, int p_num)
        :   Receive_port(p_name, p_num), Port_type<T>()
        { class_name = "Receive";
          Set_data_connection_type(0); // specify Brokenline as default
        }
  //////////////////////////////////////////////////////////////////////
  Receive(void)
        :   Receive_port(), Port_type<T>()
        { class_name = "Receive";
          Set_data_connection_type(0); // specify Brokenline as default
        }
  //////////////////////////////////////////////////////////////////////
  ~Receive(void) {}

};


// definition of Send<T>

/*----------------------------------------------------------------------
CLASS
    Send

DESCRIPTION

    Send&lt T&gt is a class of port which sends data of type Data&lt
    T&gt.  Send&lt T&gt can be connected Receive&lt T&gt, Relay&lt T&gt,
    O_file&lt T&gt.

----------------------------------------------------------------------*/

template <class T>
class Send : public Send_port, public Port_type<T> {
    /*

    Send<T> is a class of port which sends data of type Data<T>.
    Send<T> can be connected Receive<T>, Relay<T>, O_file<T>.

    */

//  Connection function
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data_pack_type which expresses the type of data to be
  // receided.
  //
  char* Data_pack_type(void) { return data_pack_type; }

//  Functions for sending data
public:
  //////////////////////////////////////////////////////////////////////
  virtual void Send_data(time_t t, const T& v);
        /*
            Send data whose data time is t and data value is v.
            const was added by Shiiba.
        */

  //////////////////////////////////////////////////////////////////////
  virtual void Send_by_string(char*& data_str);
        /*
            Send data of type Data<T> whose value is stored in data_str.
        */

//  Initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new Send&lt T&gt Port
  virtual Send<T>* NewPort() { return new Send<T>(); }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
        { return Send_port::Init(p_name, p_num); }
  //////////////////////////////////////////////////////////////////////
  Send(char* p_name, int p_num)
        :   Send_port(p_name, p_num), Port_type<T>()
        { class_name = "Send"; }

  //////////////////////////////////////////////////////////////////////
  Send(void)
        :   Send_port(), Port_type<T>()
        { class_name = "Send"; }

  //////////////////////////////////////////////////////////////////////
  ~Send(void) {}

};


//  Definition of Relay<T>

/*----------------------------------------------------------------------
CLASS
    Relay

    Relay&lt T&gt is a class of port which relays Data&lt T&gt.

DESCRIPTION

    Relay&lt T&gt is a class of port which relays Data&lt T&gt.

    Relay&lt T&gt can be connected with Receive&lt T&gt, Send&lt T&gt,
    Relay&lt T&gt, and O_file&lt T&gt.

----------------------------------------------------------------------*/

template <class T>
class Relay : public Relay_port, public Port_type<T> {
    /*

    Relay<T> is a class of port which relays Data<T>.

    Relay<T> can be connected with Receive<T>, Send<T>, Relay<T>, and
    O_file<T>.
    */

//  Connection
public:
  //////////////////////////////////////////////////////////////////////
  char* Data_pack_type(void) { return data_pack_type; }

//  Initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new Relay&lt T&gt Port
  virtual Relay<T>* NewPort() { return new Relay<T>(); }

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
        { return Relay_port::Init(p_name, p_num); }
  //////////////////////////////////////////////////////////////////////
  // Constructor
  Relay(char* p_name, int p_num)
        :   Relay_port(p_name, p_num), Port_type<T>()
        { class_name = "Relay"; }
  //////////////////////////////////////////////////////////////////////
  // Constructor
  Relay(void)
        :   Relay_port(), Port_type<T>()
        { class_name = "Relay"; }
  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~Relay(void) {}

};


//  Definition of I_file<T>

/*----------------------------------------------------------------------
CLASS
    I_file

    I_file&lt T&gt is a class of input port which reads in Data&lt T&gt
    from an input file and sends them.

DESCRIPTION
    I_file&lt T&gt is a class of input port which reads in Data&lt T&gt
    from an input file and sends them.

    I_file&lt T&gt can be connected with Receive&lt T&gt, Relay&lt T&gt,
    O_file&lt T&gt.

    I_file&lt T&gt reads a input file using "simple data format for
    input and output."

    "Simple data format for input and output" is a format in which

    EXAMPLE
        all the data items for a datapack should be written in one line,
        and between data items should be separated with a number of
        spaces or tabs.

        Example:    120 1.2 2.3 4.5
    END

    If you need different format, please make a derived class of
    I_file&lt T&gt and redefine Make_data_pack.

----------------------------------------------------------------------*/


template <class T>
class I_file : public Input_file, public Port_type<T> {
    /*

    I_file<T> is a class of input port which reads in Data<T> from an
    input file and sends them.

    I_file<T> can be connected with Receive<T>, Relay<T>, O_file<T>.

    I_file<T> reads a input file using "simple data format for input and
    output."

    "Simple data format for input and output" is a format in which

        all the data items for a datapack should be written in one line,
        and between data items should be separated with a number of
        spaces or tabs.

        Example:    120 1.2 2.3 4.5

    If you need different format, please make a derived class of
    I_file<T> and redefine Make_data_pack.

    */

//  Data member

//  Connection
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data_pack_type which expresses the type of data to be
  // receided.
  //
  char* Data_pack_type(void) { return data_pack_type; }

//  Data input
public:
  //////////////////////////////////////////////////////////////////////
  // Reads string data of Data&lt T&gt, creates Data&lt T&gt in heap
  // memory, and returns the pointer to the heap memory.
  //
  void* Make_data_pack(char* line_data);
        /*

        Reads string data of Data<T>, creates Data<T> in heap memory,
        and returns the pointer to the heap memory.

        */

public:
  //////////////////////////////////////////////////////////////////////
  // Sends Data&lt T&gt which is stored in the form of string.
  virtual void Send_by_string(char*& string);
        /*

        Sends Data<T> which is stored in the form of string.

        */

//  Initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new I_file&lt T&gt Port
  virtual I_file<T>* NewPort() { return new I_file<T>(); }

  //////////////////////////////////////////////////////////////////////
  // Init
  Boolean Init(char* f_name) { return Input_file::Init(f_name); }

  //////////////////////////////////////////////////////////////////////
  // Constructor
  I_file(char* f_name)
        :   Input_file(f_name), Port_type<T>()
        { class_name = "I_file"; }
  //////////////////////////////////////////////////////////////////////
  // Constructor
  I_file(void)
        :   Input_file(), Port_type<T>()
        { class_name = "I_file"; }
  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~I_file(void) {}

};


//  Definition of O_file<T>

/*----------------------------------------------------------------------
CLASS
    O_file

DESCRIPTION
    O_file&lt T&gt is a class of output ports which receive Data&lt T&gt
    and print them in a file.

    O_file&lt T&gt receives Data&lt T&gt.
    O_file&lt T&gt can be connected with Send&lt T&gt, Relay&lt T&gt,
    and I_file&lt T&gt.

    O_file&lt T&gt prints data of type Data&lt T&gt using
    Data&lt T&gt::Dpack2ofile(FILE* fp).

----------------------------------------------------------------------*/

template <class T>
class O_file : public Output_file, public Port_type<T> {
    /*
    O_file<T> is a class of output ports which receive Data<T> and print
    them in a file.

    O_file<T> receives Data<T>.
    O_file<T> can be connected with Send<T>, Relay<T>, and I_file<T>.

    O_file<T> prints data of type Data<T> using
    Data<T>::Dpack2ofile(FILE* fp).

    */

// Data member

// For connection

public:
  //////////////////////////////////////////////////////////////////////
  // Returns the data_pack_type which expresses the type of data to be
  // receided.
  //
  char* Data_pack_type(void) { return data_pack_type; }

//  Output of data

public:
  //////////////////////////////////////////////////////////////////////
  void Make_string_data(char*, void*) {};

  //////////////////////////////////////////////////////////////////////
  // Though this function is defined in the base class Output_file,
  // it is redifined in O_file&lt T&gt.
  virtual void Output_data(void);
    /*
    #  O_file<T>::Output_data(void)

    Though this function is defined in the base class Output_file,
    it is redifined in O_file<T>.

    */

//  Initialization, constructor, and destructor
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new O_file&lt T&gt Port
  virtual O_file<T>* NewPort() { return new O_file<T>(); }

  //////////////////////////////////////////////////////////////////////
  // Init
  Boolean Init(char* f_name, int l_len = 128)
        { return Output_file::Init(f_name, l_len); }
  //////////////////////////////////////////////////////////////////////
  // Constructor
  O_file(char* f_name, int l_len = 128)
        :   Output_file(f_name, l_len), Port_type<T>()
        { class_name = "O_file"; }
  //////////////////////////////////////////////////////////////////////
  // Constructor
  O_file(void)
        :   Output_file(), Port_type<T>()
        { class_name = "O_file"; }
  //////////////////////////////////////////////////////////////////////
  // Destructor
  ~O_file(void) {}

};

#endif  // ! p_tmp_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.21 $
Changed in Revision: 1.20
The concept of "data connection type" was introduced into Receive<T>.

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */
