/*;  relay.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision$$ {\rhead{\sl relay.h\quad\thepage}} */
/*;  $$Date$$  */
/*;  $$Source$$  */
/*;  $$RCShost$$  */

// @(#) relay.h -- Base class for creating relay ports

/*
{\ {\setlength{\parskip}{4pt}% latex-mode
...
}% end of latex-mode }
*/

#if !defined(relay_h)
#define relay_h

#include <stdio.h>
#include <time.h>           //; time_t , tm 

#include "common.h"         //; ץˤ붦
#include "list.h"           //; 饹 List 
#include "sender.h"         //; 饹 Sender_port 


//; 饹 Relay_port .

/*----------------------------------------------------------------
CLASS

    Relay_port

KEYWORDS

    relay, port

DESCRIPTION
    The Relay_port class is a model of relay ports.

    The Relay_port has a data member "sender". "sender" is used to store
    the pointer to the sender which sends data packs to the current
    port.

    Member functions for

    <ul>
      <li>  Connecting the current port with the port which sends data
            packs to the current port
      <li>  Connecting the current port with the port which receives data
            packs from the current port
    </ul>

    are defined.

    Only one port can be connected to the port for receiving data packs.
    More than one ports can be connected to the ports which sending data
    packs.

    <japanese>
    ܷüҥǥΥ饹

    饹 Relay_port Ǥ,
    <pre>
        ǡѥåü    : sender
    </pre>
    򵭲,
    <pre>
        üҤȤ³ǽ
        üҤȤ³ǽ
    </pre>
    ޤ. üҤȤϡüҤ˥ǡäƤ¦
    üҤǤüҤȤϡüҤǡȤ¦
    üҤǤ

    Relay_port ³ǤüҤϰĤǤ.
    Relay_port ˤ, ʣüҤ³Ǥޤ.

    </japanese>

----------------------------------------------------------------*/

class Relay_port : public Sender_port {

//; 0.  ǡ.
protected:
  //////////////////////////////////////////////////////////////////////
  // Address of the sender which sends data packs to this Relay_port
  // casted to type void*
  //
  // <japanese>
  // void* ˥㥹Ȥ줿üҤΥɥ쥹
  // </japanese>
  void* sender;

//; 1.  ³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the string which shows the type of data packs which are
  // treated by this Relay_port.
  //
  // In derived classes of the class Relay_port, this funstion should be
  // defined so that it returnts the data_pack type string. For example,
  // for the Relay_port which treats int, define as
  // <pre>
  // char* Data_pack_type(void) { return "INT"; }
  // </pre>
  //
  // <japanese>
  // ѥǡѥåɽʸ֤
  //
  // 쥯饹 Sender_port ν貾۴ؿǤ.
  //  Relay_port Ǥ貾۴ؿȤƤޤ.
  // Relay_port 饹, ѥǡѥåɽʸؤΥݥ
  // 󥿤֤褦ɬƲ.
  //
  // 㤨, ѥǡѥå "INT" Ǥ饹Ǥ,
  // <pre>
  // char* Data_pack_type(void) { return "INT"; }
  // </pre>
  // Τ褦ޤ.
  //
  // ³ˤ, ³üҤȤüҤ Data_pack_type פ뤳
  // Ȥǧޤ.
  //
  // <pre>
  //    RETURN:
  //        char*     ѥǡѥåɽʸؤΥݥ.
  // </pre>
  // </japanese>
  virtual char* Data_pack_type(void) = NULL;

public:
  //////////////////////////////////////////////////////////////////////
  // Sets the sender of this Relay_port to <b>s</b>
  //
  // [in] s - address of the sender casted to void*
  //
  // Returns YES if the setting was successfull. For the following cases,
  // it returns NO.
  // <ul>
  // <li> <b>s</b> is NULL.
  // <li> already the sender has been set.
  // <li> the type of datapacks which s treats is different from that of
  //      datapacks which this ports treats
  // </ul>
  Boolean Set_sender(void* s);
        /*
          Relay_port::Set_sender(void*) (virtual)     üҤ

        ü Relay_port::sender  s ꤷ, YES ֤ޤ.

        ξ, 顼åɽ, NO ֤ޤ.
            s  NULL ξ.
            üҤü sender Ѥߤξ.
            üҤ s , Data_pack_type ۤʤ.

        ARGUMENT:
            s     void* ˥㥹Ȥ줿üҤΥɥ쥹.
        RETURN:
            YES   ü sender ꤵ줿.
            NO    s  NULL ξ.
                    üҤü sender Ѥߤξ.
                    üҤ s , Data_pack_type ۤʤ.
        ACCIDENT:
            s  Port* ѿǼƤʤ.
        */
public:
  //////////////////////////////////////////////////////////////////////
  // Reports whether this Relay_port is ready or not.  When "sender" and
  // "number_of_receivers" have been defined, it returns YES, otherwise
  // it returns NO.
  //
  Boolean Are_you_ready(void)
        { return ((sender != NULL) && (number_of_receivers >= 0))
                    ? YES : NO; }
        /*
          Relay_port::Are_you_ready(void) (virtual, inline)
                ׻γǧ

        ǡѥåѲǽʾ֤Ǥ뤳Ȥǧޤ.
        sender ꤵƤ, sender ǡѥåѤǤޤ.
        number_of_receivers  0 ʾʤ, receiver_array Ѥߤ,
        üҤ˥ǡѥåǤޤ.

        RETURN:
            YES   sender, receiver_array Ѥߤξ.
            NO    sender, receiver_array ξʤȤ̤ξ.
        */

//  2.  ǡѥåѵǽ, õǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Orders the sender to create a data_pack array.
  void Set_data_pack_array(void)
        { ((Port*) sender)->Set_data_pack_array(); }
        /*
          Relay_port::Set_data_pack_array(void) (virtual, inline)
                ǡݥѥåκ

        쥯饹 Sender_port ν貾۴ؿǤ.
        sender ˥ǡѥå褦̿ᤷޤ.
        */
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the oldest data time of the data stocked in the sender.
  time_t Oldest_data_time(void)
        { return ((Port*) sender)->Oldest_data_time(); }

  //////////////////////////////////////////////////////////////////////
  // Returns the newest data time of the data stocked in the sender.
  time_t Newest_data_time(void)
        { return ((Port*) sender)->Newest_data_time(); }

  //////////////////////////////////////////////////////////////////////
  // Returns the number of datapacks which stored in the sender
  int Number_of_data_packs(void)
        { return ((Port*) sender)->Number_of_data_packs(); }

  //////////////////////////////////////////////////////////////////////
  // Reports whether this Relay_port can get the data from <b>t_from</b>
  // to <b>t_to</b>
  //
  //  [in] t_from  - start time
  //  [in] t_to    - end time
  //
  Boolean Can_you_get_data(time_t t_from, time_t t_to)
        { return ((Port*) sender)->Can_you_get_data(t_from, t_to); }
        /*
          Relay_port::Can_you_get_data(time_t,time_t)  (virtual,inline)
                ǡѥå˴ؤǧ

        쥯饹 Sender_port ν貾۴ؿǤ.
        sender 䤤碌, t_from  t_to ʹߤޤǤΥǡѥ
        ǽǧޤ.
        ǽʤ YES ֤, Բǽʤ NO ֤ޤ.

        δؿüҤ Can_you_get_data ƤӽФޤ.
        ˤüҤǥǡѥåǤ뤫ǧǤޤ.

        ARGUMENT:
            t_from    t_from ǡѥåǽǤ뤳Ȥ
                        ǧ.
            t_to      t_to ʹߤޤǥǡѥåǽǤ뤳Ȥ
                        ǧ.
        RETURN:
            YES       t_from  t_to ʹߤޤǤΥǡѥå
                        ǽʾ.
            NO        t_from  t_to ʹߤޤǤΥǡѥå
                        Բǽʾ.
        ACCIDENT:
            ³, sender  NULL ξ.
              δؿ, ׻ˤѤǽΤ, 
               sender  NULL ǤʤȤǧޤǤ.
            sender  Port* ʳѿǼƤ.
        */

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // üҤؤη׻ϡüҤž
  // </japanese>
  void Creq(void) { ((Port*)sender)->Creq(); }

public:
  //////////////////////////////////////////////////////////////////////
  // Get datapack array <b>dp_array</b> and the number of datapacks
  // <b>num_of_fp</b> from the sender.
  //
  //  [out] dp_array  - datapack array
  //  [out] num_of_dp - the number of datapacks
  //
  void Get_data_pack_array(void**& dp_array, int& num_of_dp)
        { ((Port*) sender)->Get_data_pack_array(dp_array, num_of_dp); }
        /*
          Relay_port::Get_data_pack_array(void**&,int&) (virtual,inline)
                ǡѥåμ

        쥯饹 Sender_port ν貾۴ؿǤ.
        ü sender , ǡѥåޤ.

        dp_array ˤ, void* ˥㥹Ȥ줿 Data_pack* ѿ
        Ǽ, num_of_dp ˤ dp_array ǿǼޤ.

        δؿüҤ Get_data_pack_array ƤӽФޤ.
        ˤüҤǥǡѥåǤޤ.

        ARGUMENT:
            dp_array      ѥǡѥåؤΥݥ󥿤Ǽ뻲
                            Ȱ.
            num_of_dp     dp_array ǿǼ뻲Ȱ.
        ACCIDENT:
            ³, sender  NULL ξ.
              δؿ, ׻ˤѤǽΤ, 
               sender  NULL ǤʤȤǧޤǤ.
            sender  Port* ʳѿǼƤ.
        */
public:
  //////////////////////////////////////////////////////////////////////
  // Get two pointers to datapacks <b>former</b> and
  // <b>later</b>. <b>former</b> is
  // the pointer to the newest datapack before time
  // <b>data_time</b>. <b>later</b> is the pointer to the
  // oldest datapack after <b>data_time</b>.
  // <b>flag</b> specifies the option
  // <ul>
  //  <li> 'b' -- get both of <b>former</b> and <b>later</b>
  //  <li> 'f' -- get <b>former</b> at least
  //  <li> 'l' -- get <b>later</b> at least
  // </ul>
  //
  // When the pointer(s) specified by <b>flag</b> can be got, it returns
  // YES, otherwise NO.
  //
  // [out] former - pointer to a datapack
  // [in] data_time - specified time
  // [out] later  - pointer to a datapack
  // [in] flag  - option to get pointers to datapacks
  //
  Boolean Get_data_pack(void*& former, time_t data_time, void*& later,
                          char flag = 'b')
        { return ((Port*) sender)
                    ->Get_data_pack(former, data_time, later, flag); }
        /*
          Relay_port::Get_data_pack(void*&,time_t,void*&,char) (virtual)
                ǡѥåμ

        쥯饹 Sender_port ν貾۴ؿǤ.

        ꤷ, sender ǡѥåޤ.

        former ˤ, ǡϿ郎 data_time  data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        later ˤ, ǡϿ郎 data_time ʹߤ data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        ǡϿ郎 data_time ǤǡѥåϿƤ
        , former, later Ȥ⤽Ǽޤ.

        ǡѥåǤʤ, former, later  NULL
        ꤵޤ.

        flag ϼΥ⡼ɤ򤹤뤿Υץ, ̤Ǥ.
            'b'       former, later ξɬ.
                        Ǥʤ, ξȤ NULL ꤹ.
            'f'       later ǤʤƤ, former Τ߼.
                        later Ǥʤ, NULL ꤹ.
            'l'       former ǤʤƤ, later Τ߼.
                        former Ǥʤ, NULL ꤹ.
            ¾    former, later ΤʤȤ.
                        ǤʤΤ NULL ꤹ.

        flag Υǥեͤ, 'b' Ǥ.
        ޤ,  later Ϥʤ, flag  f Ȥ former Τ
        ,  former Ϥʤ, flag  l Ȥ later 
        ߼ޤ.

        δؿüҤ Get_data_pack ƤӽФޤ.
        ˤüҤǥǡѥåǤޤ.

        ARGUMENT:
            former        d_time  d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            data_time     . (sec)
                            λ˺ǤᤤǡϿļϿ
                            ѥå.
            later         d_time ʹߤ d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            flag          ǡѥåμꤹե饰.
        RETURN:
            YES           ΥǡѥåǤ.
            NO            data_pack_array  NULL ξ.
                            ΥǡѥåǤʤ.
        */
public:
  //////////////////////////////////////////////////////////////////////
  // Forwards the message Reduce_data_pack_stock() to the sender
  void Reduce_data_pack_stock(void)
        { ((Port*) sender)->Reduce_data_pack_stock(); }
        /*
          Relay_port::Reduce_data_pack_stock(void) (virtual,inline)
                ץǡѥåξõ

        ü sender ץǡѥåõ褦ؼޤ.

        üҤ owner Ƿ׻λѤ, ץޤ.

        δؿüҤ Get_data_pack ƤӽФޤ.
        ˤüҤǥǡѥåǤޤ.
        */

//  3.  ϵǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Prints the information about connection of this Relay_port.
  // This function is used for debugging.
  void Print_connection(FILE* fp_out = stderr, char* head = "    ");
        /*
          Relay_port::Print_connection(FILE*,char*) (virtual)
                ³ν

        üҤ³ط˴ؤ fp_out ˽Ϥޤ.
        ۴ؿʤΤ, 饹ǺǤޤ.

        ARGUMENT:
            fp_out    եؤΥݥ. ǥե stderr.
            head      ϻΥإå. ǥեͣ.
        */

//  4.  󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  // Sets the object name to <b>p_name</b> and the object number
  // to <b>p_num</b>
  Boolean Init(char* p_name, int p_num)
        { return Port::Init(p_name, p_num); }

  //////////////////////////////////////////////////////////////////////
  // Constructor.
  // Sets the object name to <b>p_name</b> and the object number
  // to <b>p_num</b>
  Relay_port(char* p_name, int p_num);

  //////////////////////////////////////////////////////////////////////
  // Constructor.
  Relay_port(void);

  //////////////////////////////////////////////////////////////////////
  // Destructor.
  ~Relay_port(void);
        /*
          Relay_port::Init(char*,int) (inline,virtual)    ؿ
          Relay_port::Relay_port(char*,int)       󥹥ȥ饯
          Relay_port::Relay_port(void)            󥹥ȥ饯
          Relay_port::~Relay_port(void) (virtual)     ǥȥ饯

        Relay_port(char*,int) , 쥯饹 Sender_port Υǥեȥ
        ȥ饯Ѥ, Relay_port::Init Ѥ, Фʲ̤
        ޤ.
            class_name            : "Relay_port"
            object_name           : p_name (ҡפ˥ԡ)
            object_number         : p_num
            owner                 : NULL
            sender                : NULL
            buffer_list           : ҡפ˳ݤ. (³)
            receiver_array        : NULL
            number_of_receivers   : -1

        ǥեȥ󥹥ȥ饯, Relay_port::Init ¹Ԥޤ.
        η, ʲȤʤޤ.
            object_name           : NULL
            object_number         : -1
        ¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
        ξ, Relay_port::Init ѤƥФƲ.

        ǥȥ饯, ܴ쥯饹 Object  virtual Ƥޤ.

        ARGUMENT:
            p_name    ü̾.
            p_num     üֹ.
        EXIT:
            5         object_name εΰݤǤʤ.
                        (Object::Init)
            5         buffer_list εΰݤǤʤ.
                        (Sender_port::Sender_port)
        */

};


#endif  //  ! relay_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.4 $
Revision 1.4: alterations from Revision 1.3
Definition of Number_of_data_packs was added.

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
