/* input_f.cc {\documentstyle{jsrc2latex}\rhead{\sl input\_f.cc\quad\thepage}
   \font\tenrm=cmtt10}Implementaion of Input_file
{\ \begin{verbatim}
$Revision: 1.1 $ $Date: 2000/12/02 10:10:15 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode

}% end of latex-mode }
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>			// definition of time_t, tm
#include <limits.h>			// definition of LONG_MIN

#if defined(__MSDOS__)		// if ths OS is MS-DOS
#include <sys\stat.h>		//
#elif !defined(__MSDOS__)	// else
#include <sys/stat.h>		//
#endif

#include "common.h"
#include "port.h"
#include "input_f.h"


//	Implementation of class Input_file

void Input_file::Input_data(void)
{	//		Input_file::Input_data(void) (define)

	Set_data_pack_array();

	// This member function is implemented in the base class
	// Send_port. This funcition add the datapacks which may remain in
	// the list to the array.

	Reduce_data_pack_stock();

	// This member function is implemented in the base class
	// Send_port. This funcition delete the unnecessary datapacks.

    // The above two member functions work well even if they are
	// executed when no data have been read in.

	struct stat buf;
	if (stat(object_name, &buf) != 0)
		{ Eprint(__FILE__, __LINE__); Oprint(); return; }
			// Input_file::Input_data(void)
			// Cannot get status of 'input_file'.

	if (buf.st_mtime <= file_time_at_input) return;
		// the input file was not updated.

    file_time_at_input = buf.st_mtime;

	FILE* fp = fopen(object_name, "r");
	if (fp == NULL) {
		Eprint_fopen(__FILE__, __LINE__, object_name, "r");
			// Input_file::Input_data(void)
		Oprint(); exit(3);
	}

	// calculate maximum line length
	char buffer;
	int c, c_max;
	for (c = 0, c_max = -1; (buffer = fgetc(fp)) != EOF;) {
		c++;
		if (buffer == '\n') { c_max = (c_max > c) ? c_max : c; c = 0; }
	}
	c_max = (c_max > c) ? c_max : c;
	if (c_max <= 0)
		{ fclose(fp); Eprint(__FILE__, __LINE__); Oprint(); return; }
			// Input_file::Input_data(void)
			// Cannot send input data from this 'input_file'.

	char* line_data = new char[c_max + 1];
	if (line_data == NULL) {
		fclose(fp);
		Eprint_memory(__FILE__, __LINE__, "char", (c_max + 1));
			// Input_file::Input_data(void)
			// For buffer string to read 'input_file'.
		Oprint(); exit(5);
	}

	fseek(fp, 0, 0);	// return to top
    while (fgets(line_data, c_max+1, fp) != NULL) {
        if (line_data[0] == '#') continue;    // skip comment lines
        c = 0;
        while (line_data[c] == ' ' || line_data[c] == '\t') c++;
        if (line_data[c] == '\n') continue;
        Stock(Make_data_pack(line_data));
    }
	fclose(fp);

	delete [] line_data;
	Set_data_pack_array();

	// print newest data time
	if (Number_of_data_packs() > 0) {
		fprintf(stderr, "File: %s  newest data time : %ld\n",
				object_name, Newest_data_time());
	}

	return;
}


void Input_file::Cprint(FILE* fp_out, char* head)
{	//		Input_file::Cprint(FILE*,char*) (define)

	fprintf(fp_out,
		"%sInput_file : %s<%s> %s \n",
		head, class_name, Data_pack_type(), object_name);
}

// modified by Suzuki,T (Sat Feb 10 15:44:07 1996)
void Input_file::Oprint(FILE* fp_out, char* head)
{
    Cprint(fp_out, head);
}

Input_file::Input_file(char* f_name)
	:	Input_port(),
		file_time_at_input(LONG_MIN)
{	//		Input_file::Input_file(char*) (define)

	class_name  = "Input_file";

	Init(f_name);
}


Input_file::Input_file(void)
	:	Input_port(),
		file_time_at_input(LONG_MIN)
{	//		Input_file::Input_file(void) (define)

	class_name  = "Input_file";
}


Input_file::~Input_file(void)
{	//		Input_file::~Input_file(void) (define)
}

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $
Changes in Revision: 1.8
Member function Input_data prints the newest data time after
it has read the data file.

Changes in Revision: 1.7
The buffer length bug fix when calling
fgets(line_data, c_max+1, fp)

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */
