/*;  input_f.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl input\_f.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 10:01:16 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/input_f.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

//; @(#) ϥեüҥǥκ

#if !defined(input_f_h)
#define input_f_h

#include <stdio.h>
#include <time.h>       //; time_t , tm 

#include "object.h"     //; 饹 Object 
#include "input.h"      //; 饹 Input_port 

#include <iostream>

/*----------------------------------------------------------------
CLASS

    Input_file

    Class of port models of input files

DESCRIPTION

    An object of Class Input_file stores

    <dl><dd>
    <dl><dt> Object::object_name
          <dd> input file name
        <dt> file_time_at_input
          <dd> final update time of the input file
    </dl></dl>

    and has

    <dl><dd>
    function to read in data and send them as datapacks to the
    send port
    </dl>

    <japanese>
    ϥեüҥǥ, üҤΤߤǥǥǤ
    ᤹뤳ȤǤޤʤ, Υǥ, ե뤫ǡ
    ɤ߹,ǡѥåѴüҤ˥ǡѥå򥹥ȥå
    

    ºݤˤ, ϥեüҥǥ, ǥǥΰĤȤƥ
    벽줺, üҥǥǥȤƼ¸Ƥޤ
    , üҥǥ˥եɤߤȤ굡ǽɲä, ϥե
    üҥǥ뤬Ƥޤ饹 Input_file ˴Ϣ
    ϤĤΤ褦Ǥ

    <pre>

        Object <-- Port <--+
                           |
     +---------------------+
     |
     +---   Send_port <-- Input_port <-- Input_file   <--+-- I_file<T>
                                                         |
                                         Port_type<T> <--+
    </pre>
    <pre>

    饹 Input_file Ǥ,
        ϥե̾             : Object::object_name
        ϥեκǽ : file_time_at_input
    򵭲,
        ϥե뤫Υǡѥåǽ
    ޤ.
    </pre>
    </japanese>

----------------------------------------------------------------*/


//;  饹 Input_file .

class Input_file : public Input_port {

//;  0.  ǡ.
protected:
  ////////////////////////////////////////////////////////////////
  // final update time of the input file
  //
  // <japanese>
  // ϥեκǽ
  //
  // 餷ǡ줿ȤˡƱ̾ϥեե
  // ɲý񤭤뤫ѤꤷƤޤΤ
  // ˡǡɤߤȤˡեκǽϿ
  // ƤĤ˥եɤɬפȤˡե
  // ǽ򸫤ơե뤬줿褦ˤ
  // 
  //
  // </japanese>
  time_t file_time_at_input;

//;  1.  ³˴ؤ뵡ǽ.
public:
  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ǡݥѥåɽʸ֤
  //
  // Send_port ν貾۴ؿǤ, Input_file Ǥ貾۴ؿȤ
  // ޤ. Input_file 饹, ǡѥåɽʸ
  // ؤΥݥ󥿤֤褦ɬƲ.
  //
  // 㤨, ǡѥå "INT" Ǥ饹Ǥ,
  // char* Data_pack_type(void) { return "INT"; }
  // Τ褦ޤ.
  //
  // ³, 󥯻ˤ, üҤ Data_pack_type , üҤ
  // Data_pack_type Ȥפ뤳Ȥǧޤ.
  //
  // <pre>
  // RETURN:
  // char*     ǡѥåɽʸؤΥݥ.
  // </pre>
  // </japanese>
  virtual char* Data_pack_type(void) = NULL;

//;  2.  ϥեǡǽ.
public:
  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ʸǡǡѥå롣貾۴ؿ
  // 饹Ǽ뤳Ȥ׵ᤵƤޤ
  //
  // ʬʸǡ򣱸ĤΥǡѥåѴؿ饹
  // Ʋ.
  //
  // ɸŪ饹Ǥ, ǡѥå˱Ƽμ³
  // Ʋ.
  //
  // <pre>
  // (1) ѥǡѥåεΰҡפ.
  // (2) ʸ line_data ǡϿ, ǡͤɤ߼.
  // (3) (2) ɤ߼äͤ (1) ΥǡѥåΥФ.
  // (4) (1) ΥǡѥåΥݥ󥿤 void* ˥㥹Ȥ֤.
  // </pre>
  //
  // 貾ؿʤΤ, 饹ɬƲ.
  //
  // <pre>
  // ARGUMENT:
  //  line_data     ϥǡʬʸ.
  // RETURN:
  //  void*         ǡѥåؤΥݥ󥿤 void* ˥
  //                  Ȥ.
  // </pre>
  // </japanese>
  virtual void* Make_data_pack(char* line_data) = NULL;

  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ϥեɤ߼, Input_file::Make_data_pack ˤǡ
  //  å, ǡѥå Send_port::Stock 
  // Ѥޤ.
  //
  // </japanese>
  // ϥեɤ߼, Input_file::Make_data_pack ˤǡ
  // å, ǡѥå Send_port::Stock Ѥ
  // ޤ.
  //
  // ǹƬ '\n', '#' ξ, , ȹԤȸʤޤ.
  //
  // եޤǽλ, Send_port::Send_data_pack_array
  // Ѥ data_pack_array ޤ.
  //
  // , ϥեκǽѹ郎 file_time_at_input ξ
  // ϲ⤻ return ޤ.
  // 󥹥ȥ饯 file_time_at_input  LONG_MIN ꤵΤ,
  // Ϥɬ¹Ԥޤ.
  //
  // <pre>
  // EXIT:
  // Level 3   ϥե object_name 򥪡ץǤʤ.
  // Level 5   ɤ߼ʸҡפ˳ݤǤʤ.
  // ACCIDENT:
  // ǡεҤ˸꤬.
  // ؿ Input_file::Make_data_pack μ˸꤬.
  // </pre>
  // </japanese>
  virtual void Input_data(void);

  //////////////////////////////////////////////////////////////////////
  // When the input port receives a calculation request, throw 99. This
  // will be caught in the catch block in Total_system::Work().
  // See total.cc.
  void Creq(void) {
    fprintf(stderr, "Cannot get data from %s \n", object_name);
    throw 99;
  }

//;  3.  ɽǽ.
public:
  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥Ⱦν (owner ξޤޤ)
  //
  // ϥե̾ȤüҤüҤξե fp
  // ˽Ϥޤ.
  //
  // ؿ Cprint ϥ饹 Object β۴ؿʤΤ, 饹Ȥѹ
  // Ǥޤ.
  //
  // Object::Cprint ¿Ƥ뤿, fp_out Ϥʤ
  //  stderr ˽Ϥ, λ, head Ϥʤ head 
  // ꤵޤ.
  //
  // <pre>
  // ARGUMENT:
  //   fp_out    ֥ȾϤեؤΥݥ.
  //               Ϥʤ stderr Ȥʤ.
  //   head      ʸؤΥݥ.
  //               fp_out ϤʤΥǥեͤϣ.
  // </pre>
  // </japanese>
  void Cprint(FILE* fp_out, char* head);

  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥Ⱦν (owner ξޤ)
  // </japanese>
  void Oprint(FILE* fp_out = stderr, char* head = "    ");

//;  4.  ǽ, 󥹥ȥ饯, ǥȥ饯.
public:
  ////////////////////////////////////////////////////////////////
  // <japanese>
  // ǽ.
  //
  //  f_name, 0 Ȥ, Input_port::Init ¹Ԥޤ. ü
  // ֹ, 0 ꤷޤ.
  //
  // <pre>
  // RETURN
  //   YES       
  //   NO        ʤä
  // ARGUMENT:
  //   f_name    ϥե̾.
  // EXIT:
  //   5         object_name εΰ褬ݤǤʤ.
  // </pre>
  //
  // </japanese>
  Boolean Init(char* f_name) { return Input_port::Init(f_name, 0); }

public:
  //////////////////////////////////////////////////////////
  // <japanese>
  // 󥹥ȥ饯ϡե̾
  //
  // Input_port(char*) , 쥯饹 Input_port Υǥեȥ󥹥
  // 饯Ѥ, Input_file::Init Ѥ, Фʲ̤
  // ޤ.
  //
  // <pre>
  // class_name            : "Input_file"
  // object_name           : f_name (ҡפ˥ԡ)
  // object_number         : 0
  // owner                 : NULL
  // buffer_list           : ҡפ˳ݤ. (³)
  // receiver_array        : NULL
  // number_of_receivers   : -1
  // data_pack_array       : NULL
  // number_of_data_packs  : -1
  // file_time_at_input    : LONG_MIN
  // </pre>
  // </japanese>
  Input_file(char* f_name);

  //////////////////////////////////////////////////////////////
  // <japanese>
  // 󥹥ȥ饯ɬפʾϡǡInit Ϳ뤳Ȥˤơ
  // Ȥꤢ construct 뤿ΤΡ
  // </japanese>
  Input_file(void);

  ////////////////////////////////////////////////////////////////
  // Destructor
  ~Input_file(void);

};

#endif  //  ! input_f_h

/*; end of file */
