/*;  object.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl object.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 09:54:31 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/object.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) objext.h: heafer file for class Object

/*
{\ {\setlength{\parskip}{4pt}% latex-mode
...
}% end of latex-mode }
*/

#if !defined(object_h)
#define object_h

#include <stdio.h>
#include <string.h>

#include "common.h"     //; ץˤ붦
#include "ohoption.h"   //; OHyMoS Υץ

//;  饹 Object .

/*----------------------------------------------------------------------

CLASS

  Object

  Base class of all ohymos classes

KEYWORDS

  object, owner, class_name, object_name, object_number

DESCRIPTION

  Base class of all ohymos classes. An element of type Object has four
  basic data members: owner, class_name, object_name, object_number.

  <dl>
  <dt> Definition of "owner".
  <dd> When an object A has an object B as its data member, we call A is
       the owner of B.  The data member "owner", a pointer of void*, is
       the pointer to the owner of the current object.

  <dt> Definition of "class_name"
  <dd> class_name stores the name of the class.  For example,
       "classname" of
       an object of the class Object is "Object".

  <dt> Definition of "object_name"
  <dd> object_name stores the name of the objcet.  User descides the
       name of each object.

  <dt> Definition of "object_number"
  <dd> object_number stores the identification number.

       object_name and object_number are used to identify ohymos
       elements owned by an ohymos element.  Suppose an ohymos element A
       owns ohymos elements e1, e2, ..., en.  Then elements e1, e2, ...,
       en should be able to be identified by the pair object_name and
       object_number.

  </dl>

----------------------------------------------------------------------*/

/* <japanese>
{\ {\setlength{\parskip}{4pt}% latex-mode

OHyMoS Υ饹ƴܷǥΥ饹 \code{Object} Ѿƺ
ޤΥإåեǤϡ饹 \code{Object} Ƥޤ

OHyMoS Υ桼ϡ̤ϡ饹 \code{Object} Ѿƺ줿
\verb|Element|  \verb|Send_port| ʤɡ饹 \code{Object} β̤Υ
饹Ѿơ桼ɬפȤ륯饹нʬǤäơ饹
\code{Object} ľܷѾƥ饹뤳ȤϤʤǤ礦

饹 \code{Object} ϡǡС

\begin{verbatim}
char* class_name;   // 饹̾
char* object_name;  // ֥̾
int object_number;  // ֹ֥
void* owner;        // void* ˥㥹Ȥ줿, ͭԤΥɥ쥹
\end{verbatim}

äƤޤOHyMoS Υ饹ƴܷ \code{Object} Ѿƺ
ΤǡOHyMoS ƤΥ󥹥󥹤ΥǡСäƤ
뤳Ȥˤʤޤ

\verb|class_name| , 饹Ȥ˶̤ʸꤷƲ. 㤨,
饹 ABC Ǥϥ󥹥ȥ饯
\begin{verbatim}
    class_name = "ABC";
\end{verbatim}

ꤷޤ.

\verb|object_name|, \verb|object_number| ˤ, Υ֥ȼȤ̾
ֹ \verb|Object::Object|, ޤ \verb|Object::Init| ꤷ
. ϤǤޤ.

\verb|owner| ˤ, ͭԤؤΥݥ󥿤 \verb|Object::Set_owner| 
ޤ. ֥ȣ֥ȣ¤ΥǡФǤ, ¤
ͭԤȸƤӤޤ. 㤨,
\begin{verbatim}
üңǥǥ¤ΥǡФǤ.
ǥǥʬϥǥ¤ΥǡФǤ.
\end{verbatim}

ΥǡСѤơ֥ȤӵǽͭԤ굡ǽ
֥ȾνϵǽʤɤƤޤ

}% end of latex-mode }
</japanese> */

class Object {
    /* <japanese>
      Object      ܷǥΥ饹

    饹 Object Ǥ,
        ֥Ȥ̾    : class_name, object_name, object_number
        ֥Ȥνͭ  : owner
    򵭲,
        ֥Ȥӵǽ
        ͭԤ굡ǽ
        ֥Ⱦνϵǽ
    ޤ.
    </japanese> */

//;  0.  ǡ, ؿ.
protected:
  //////////////////////////////////////////////////////////////////////
  // class_name stores the class name.
  char* class_name;   // class name

  //////////////////////////////////////////////////////////////////////
  // objetc_name stores the object name.
  char* object_name;  // object name

  //////////////////////////////////////////////////////////////////////
  // object_number stores the object_number.
  int object_number;  // object number

  //////////////////////////////////////////////////////////////////////
  // pointer to the "owner" of the current object casted to type void*
  void* owner;

public:

  //////////////////////////////////////////////////////////////////////
  // Returns class_name
  char* Class_name(void) { return class_name; }

  //////////////////////////////////////////////////////////////////////
  // Returns object_name
  char* Object_name(void) { return object_name; }

  //////////////////////////////////////////////////////////////////////
  // Returns object_number
  int Object_number(void) { return object_number; }

  //////////////////////////////////////////////////////////////////////
  // Returns the pointer to the owner
  void* Owner(void) { return owner; }

  /* <japanese>
          Object::Class_name(void) (inline)       class_name ֤
          Object::Object_name(void) (inline)      object_name ֤
          Object::Object_number(void) (inline)    object_number ֤
          Object::Owner(void) (inline)            owner ֤

   class_name , 饹Ȥ˶̤ʸꤷƲ.
   㤨, 饹 ABC Ǥϥ󥹥ȥ饯
            class_name = "ABC";
   ꤷޤ.

   object_name, object_number ˤ, Υ֥ȼȤֹ̾
   Object::Object, ޤ Object::Init ꤷޤ.  ϤǤޤ
   .

   owner ˤ, ͭԤؤΥݥ󥿤 Object::Set_owner ꤷޤ.  
   ֥ȣ֥ȣ¤ΥǡФǤ, ¤νͭ
   ȸƤӤޤ. 㤨,

   üңǥǥ¤ΥǡФǤ.
   ǥǥʬϥǥ¤ΥǡФǤ.
   </japanese> */

//;  1.  ̾˴ؤ.
public:

  ////////////////////////////////////////////////////////////////////////
  // If the current object has the same triplet as those given by the
  // arguments, this function returns YES.  Otherwise, NO.
  Boolean Compare(char* c_name, char* o_name, int o_num);

  //////////////////////////////////////////////////////////////////////
  // If the current object has the same object_name and object_number as
  // those given as the arguments, this function returns YES. Otherwise,
  // NO.
  Boolean Compare(char* o_name, int o_num);

  //////////////////////////////////////////////////////////////////////
  // If the current object has the same triplet as those which *obj has,
  // this function returns YES.  Otherwise, NO.
  Boolean Compare(Object* obj);

        /* <japanese>
          Object::Compare(char*,char*,int) (inline)
          Object::Compare(char*,int) (inline)
          Object::Compare(Object*) (inline)
                饹̾, ֥̾, ֹ֥

        Compare(char*,char*,int) , class_name  c_name, object_name 
        o_name, object_number  o_num ˰פ YES , פʤ
         NO ֤ޤ.

        Compare(char*,int) , object_name 
        o_name, object_number  o_num ˰פ YES , פʤ
         NO ֤ޤ.

        Compare(Object*) , class_name, object_name, object_number,
        owner ,  obj Ȱפ YES ֤, פʤä,
        ޤ obj  NULL ξ NO ֤ޤ.

        ARGUMENT:
            c_name    Ӥ륯饹̾.
            o_name    ֥̾.
            o_num     ֹ֥.
            obj       Object ֥ȤΥݥ.
        RETURN:
            YES       Υ֥ȤΥ饹̾, ֥̾, 
                        ֹ椬ͤȰפ.
                        Υ֥Ȥ obj Υ饹̾, ֥̾,
                        ֹ֥, ͭԤפ.
            NO        Υ֥ȤΥ饹̾, ֥̾, 
                        ֹ, ͭԤͤȰפʤä.
                        obj  NULL ξ.
                        Υ֥Ȥ obj Υ饹̾, ֥̾,
                        ֹ֥, ͭԤפʤä.
        </japanese> */

//;  2.  ͭԤ굡ǽ.
public:

  //////////////////////////////////////////////////////////////////////
  // Set owner to be "o".
  //
  // When already a different object has been registerd as the owner,
  // you cannot set new owner.  If this case happens, Set_owner returns
  // NO.
  Boolean Set_owner(void* o);
        /* <japanese>
          Object::Set_owner(void*)    owner 

        owner  o ꤷޤ.

        , ξ票顼åϤ NO ֤ޤ.
            o  NULL ξ. ()
            owner Ѥߤǡo ȰۤʤäƤ. ()
        λ owner ѹޤ.

        ARGUMENT:
            o     Υ֥ȤνͭԤؤΥݥ.
        RETURN:
            YES   ꤵ줿.
            NO    o  NULL ξ. ()
                    owner Ѥߤξ. ()
        </japanese> */

//;  3.  ϵǽ.
public:

  //////////////////////////////////////////////////////////////////////
  // Print "head", class_name, object_name, object_numer to the file
  // pointer fp_out
  virtual void Cprint(FILE* fp_out, char* head);

  //////////////////////////////////////////////////////////////////////
  // Print class_name, object_name, object_numer to the file pointer
  // fp_out
  void Cprint(FILE* fp_out) { Cprint(fp_out, "    "); }

  //////////////////////////////////////////////////////////////////////
  // Print class_name, object_name, object_numer to stderr
  void Cprint(char* head = "    ") { Cprint(stderr, head); }

  //////////////////////////////////////////////////////////////////////
  // Trace back the object's ancestry, that is, prints all the ancesters
  // to a total system to fp_out. "head" is printed before the ancestry.
  virtual void Oprint(FILE* fp_out, char* head);

  //////////////////////////////////////////////////////////////////////
  // Prints all the ancesters to a total system to fp_out.
  void Oprint(FILE* fp_out) { Oprint(fp_out, "    "); }

  //////////////////////////////////////////////////////////////////////
  // Prints all the ancesters to a total system to stderr.
  void Oprint(char* head = "    ") { Oprint(stderr, head); }
        /* <japanese>
          Object::Cprint(FILE*,char*) (virtual)
          Object::Cprint(FILE*)
          Object::Cprint(char*)
                ֥Ⱦν (ͭԤξ)
          Object::Oprint(FILE*,char*) (virtual)
          Object::Oprint(FILE*)
          Object::Oprint(char*)
                ֥Ⱦν (ͭԤξޤ)

        Object::Cprint Ǥ, 饹̾, ֥̾, ֹ֥
        ե fp_out ˲̤Ϥޤ.

            fprintf(fp,
                "%s[%s] : %s : %d :\n",
                head, class_name, object_name, object_number);

        Cprint(FILE*,char*) ϲ۴ؿʤΤ, 饹ȤѹǤ
        .

        Oprint Ǥ, ͭ, ͭԤνͭ, ... ˴ؤե
        fp_out ˽Ϥޤ.

        fp_out, head Υǥեͤ, stderr, ĤζʸǤ.

        ARGUMENT:
            fp_out    ֥ȾϤեؤΥݥ.
                        ǥե stderr.
            head      ʸؤΥݥ.
                        ǥեͤϣĤζʸ.
        </japanese> */

  //////////////////////////////////////////////////////////////////////
  // Returns the name which specifies the object uniquely through the
  // argument "absname"
  //
  //    "absname" will be created by concatenating
  // <pre>
  //       "absolute name of the owner"
  //       "/"
  //       object_name
  //       "__"
  //       object_number.
  // </pre>
  //     When owner == NULL, "absolute name of the owner" is taken as
  //    "" (NULL).
  //
  // Example.  Assume that the absolute name of the owner is
  //  "/total_sytetem__0", object_name of the current object is
  //  "elm_t", and object_number is 0, then "absname" is
  //  /total_system__0/elm_t__0
  //
  //    Enough space to store the resultant string should be secured
  //    by the caller of Absname.
  //
  void Absname(
    char* absname // the string to store the name
    );


//;  4.  ؿ, 󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  // Set object_name and object_number. o_name is copyed into
  // object_name, and object_number is set to o_num.
  virtual Boolean Init(char* o_name, int o_num);
        /* <japanese>
          Object::Init(char*,int) (virtual)   ؿ

        object_name, object_number ޤ.
        λ, object_name εΰϥҡפ˳ݤ,  o_name 
        ԡޤ.

        , object_name εΰ褬ݤǤʤ, 顼å
        ɽ, ٥룵 exit ޤ.

        ޤ, object_name Ѥߤλ, ȽǤ, 顼
        ɽ NO ֤ޤ.
        ξ object_name, object_number ѹޤ.

        󥹥ȥ饯, ڤӥǥեȥ󥹥ȥ饯
        ȤνѤޤ.

        ARGUMENT:
            o_name    ֥̾.
            o_num     ֹ֥.
        RETURN:
            YES       object_name, object_number ꤵ줿.
            NO        object_name, object_number Ťꤵ줿.
        EXIT:
            5         object_name εΰݤǤʤ. (new)
        </japanese> */
public:

  //////////////////////////////////////////////////////////////////////
  // Constructor. Sets object_name to o_name and object_number to o_name.
  Object(char* o_name, int o_num);

  //////////////////////////////////////////////////////////////////////
  // Constructor. Sets class_name to "Object", object_name to NULL,
  // object_number to -1, and owner to NULL.
  //
  // Normally, just after the object of classes derived from the class
  // Object has been constructed, object_number is equal to -1.  So
  // object_number can be used as a marker which shows the object is a
  // just-born baby or not.
  Object(void);

        /* <japanese>
          Object::Object  (poli-2)    󥹥ȥ饯

        Object(char*,int) , Object::Init  p_name, p_num Ϥ, 
        Фʲ̤ޤ.
            class_name        : "Object"
            object_name       : p_name (ҡפ˥ԡ)
            object_number     : p_num
            owner             : NULL

        ǥեȥ󥹥ȥ饯, ʲνԤޤ. ¾Υ
        ФФ, Ĥ󥹥ȥ饯ƱǤ.
            object_name       : NULL
            object_number     : -1
        ξ, Object::Init ѤƥФƲ.

        ARGUMENT:
            p_name    ü̾.
            p_num     üֹ.
        EXIT:
            5         object_name εΰݤǤʤ.
                        (Object::Init)
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // Destructor
  virtual ~Object(void);
        /* <japanese>
          Object::~Object  (virtual)  ǥȥ饯

        object_name  delete ޤ.

        饹 Object  virtual  ؿäƤΤǡ饹
        η˱ǥȥ饯ŬѤ褦, ۥǥȥ饯Ȥ
        .
        </japanese> */

};


//;  饹 Object Υ饤ؿμ.

inline Boolean Object::Compare(char* c_name, char* o_name, int o_num)
{   //;    Object::Compare(char*,char*,int) (define)

    if ((strcmp(object_name, o_name) != 0)
        || (object_number != o_num)
        || (strcmp(class_name, c_name) != 0)) return NO;

    //; 饹̾, ֥̾, ֹ֥椬פ.
    return YES;
}

inline Boolean Object::Compare(char* o_name, int o_num)
{   //;    Object::Compare(char*,int) (define)

    if ((strcmp(object_name, o_name) != 0)
        || (object_number != o_num)) return NO;

    //; ֥̾, ֹ֥椬פ.
    return YES;
}


inline Boolean Object::Compare(Object* obj)
{   //;    Object::Compare(Object*) (define)

    if (obj == NULL) return NO;

    if ((strcmp(obj->Object_name(), object_name) != 0)
        || (obj->Owner() != owner)
        || (obj->Object_number() != object_number)
        || (strcmp(obj->Class_name(), class_name) != 0)) return NO;

    //; 饹̾, ֥̾, ֹ֥, ͭԤפ.
    return YES;
}

#endif  // ! object_h

/*; end of file */
