/*;  jointbrl.cc {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl jointbrl.cc\quad\thepage}} */
/*;  $$Date: 2000/09/03 20:59:30 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/jointbrl.cc,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) jointbrl.cc

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "common.h"
#include "list.h"
#include "element.h"
#include "p_tmp.h"
#include "jointbrl.h"

Boolean JointBrl::Calculate(void)
{
  double sum_inflow = 0;
  for (int i = 0; i < mNumRps; i++) {
    sum_inflow += mA[i]*mRps[i].Get_data(current_time + time_step);
  }
  mSp.Send_data(current_time + time_step, sum_inflow);
  return YES;
}

void JointBrl::Register_receive_ports(void)
{
  for (int i = 0; i < mNumRps; i++) {
    Register(((void*)&mRps[i]));
  }
}

Boolean JointBrl::Init(char* oname, int onum, int numinput)
{
  Object::Init(oname, onum);
  mNumRps = numinput;
  mRps = new Receive<double>[mNumRps];
  NEWCHECK(mRps);
  mA = new double[mNumRps];
  NEWCHECK(mA);
  for (int i = 0; i < mNumRps; i++) {
    mRps[i].Init("rp", i);
  }
  return YES;
}

Boolean JointBrl::Init(char* oname, int onum, FILE* fp)
{
  Object::Init(oname, onum);

  // read the number of receive ports from from fp
  if (fscanf(fp, "%d", &mNumRps) != 1) {
    Eprint(__FILE__, __LINE__); exit(1);
  }

  mRps = new Receive<double>[mNumRps];
  NEWCHECK(mRps);
  mA = new double[mNumRps];
  NEWCHECK(mA);

  for (int i = 0; i < mNumRps; i++) {
    mRps[i].Init("rp", i);
    mA[i] = 1.0;
  }

  // read parameters
  Set_parameter(fp);

  return YES;
}

Boolean JointBrl::Initial_output(void)
{
  double sum = 0;
  for (int i = 0; i < mNumRps; i++) {
    if (mRps[i].Can_you_get_data(current_time, current_time) == NO) {
      return NO;
    }
    sum += mRps[i].Get_data(current_time);
  }
  mSp.Send_data(current_time, sum);
  return YES;
}

void JointBrl::Set_port(int numinput)
{
  mNumRps = numinput;
  if (mRps != NULL) {
    delete [] mRps;
  }
  mRps = new Receive<double>[mNumRps];
  NEWCHECK(mRps);
  mA = new double[mNumRps];
  NEWCHECK(mA);
  for (int i = 0; i < mNumRps; i++) {
    mRps[i].Init("rp", i);
  }
}

void JointBrl::Set_parameter(FILE* fp)
{
    fscanf(fp, "%d", &mDt);
    for (int i = 0; i < mNumRps; i++) {
      double a;
      fscanf(fp, "%lf", &a);
      mA[i] = a;
    }
}
/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
