/*;  scftotal.cc {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl scftotal.cc\quad\thepage}} */
/*;  $$Date: 2000/12/02 10:10:41 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/scftotal.cc,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#)

#include "scftotal.h"
#include "dfopen.h"
#include "newport.h"
#include "newelement.h"

////////////////////////////////////////////////////////////////////////
ScfTotalSystem::ScfTotalSystem(char* filename)
  : Total_system(), mRps(NULL), mSps(NULL), mElmnt(NULL), mIte(NULL),
      mScffile(NULL)
{
  class_name = "ScfTotalSystem";
  Object::Init("total_system", 0);

  // read Structure Cofiguration File
  mScffile = new char[strlen(filename) + 1];
  strcpy(mScffile, filename);

  // open the SCF
  FILE* wfp = dataFileOpen(mScffile);

  // read Numbers part
  /* Sample

    # ------------------------------------------------------------
    # Numbers part
    # ------------------------------------------------------------

    3 # number of input ports
    5 # number of components
    2 # number of iterator elements
    1 # number of iterator sets (number_of_iterartor_sets)
    1 # number of output ports

  */

  fscanf(wfp, "%d", &mNumRps);
  if (mNumRps > 0) { mRps = new Port*[mNumRps]; }

  fscanf(wfp, "%d", &mNumElmnt);
  if (mNumElmnt > 0)  { mElmnt = new Element*[mNumElmnt]; }

  fscanf(wfp, "%d", &mNumIte);
  if (mNumIte > 0)  { mIte = new Element*[mNumIte]; }

  fscanf(wfp, "%d", &number_of_iterator_sets);

  fscanf(wfp, "%d", &mNumSps);
  if (mNumSps > 0) { mSps = new Port*[mNumSps]; }

  // read Memory allocation part

  char header[21];
  char objname[21];
  char classname[21];
  char porttype[21];
  int objnum;

  // read [InputPorts] section
  /* sample

    [InputPorts]
    # idnum objnum objname classname porttype

    0  filename_a  I_file  double
    1  filename_b  I_file  double
    2  filename_c  I_file  double

  */

  fscanf(wfp, "%s", header);
  if (strcmp(header, "[InputPorts]") == 0) {
    int id;
    for (int i = 0; i < mNumRps; i++) {
      fscanf(wfp, "%d %s %s %s", &id, objname, classname, porttype);
      if (id != i) {
        fprintf(stderr, "SCF Total data error in [InputPorts] section. The id number %d was expected, but it was %d.\n", i, id);
        exit(1);
      }
      mRps[i] = new_port(classname, porttype);

	  // added by iYutaka (Thu Dec 28 11:50:19 2000)
	  // to check whether mRps[i] is NULL or not.
	  if (mRps[i] == NULL)
	  {
		  Eprint(__FILE__, __LINE__);
		  fprintf(stderr, "port %s (type %s) is not registered in portlist.h.\n",
				  classname, porttype);
		  exit(1);
	  }

      mRps[i]->Init(objname, 0);
    }
  } else {
    fprintf(stderr, "SCF Total data error: no [InputPorts] section.\n");
    exit(1);
  }

  // read [Components] section
  /*

    [Components]
    # id objname objnum classname basin_structure_file

    0   basin  0  MulKin     2
    1   basin  1  MulKin     2
    2   basin  2  ScfSubSystem  foo.scf
    3   river  0  Kinematic  2

  */
  fscanf(wfp, "%s", header);
  if (strcmp(header, "[Components]") == 0) {
    for (int i = 0; i < mNumElmnt; i++) {
      int id;
      fscanf(wfp, "%d %s %d %s", &id, objname, &objnum, classname);
      if (id != i) {
        fprintf(stderr, "SCF Total data error in [Components] section. The id number %d was expected, but it was %d.\n", i, id);
        exit(1);
      }
      mElmnt[id] = new_element(classname);

	  // added by iYutaka (Thu Dec 28 11:50:19 2000)
	  // to check whether mElmnt[id] is NULL or not.
	  if (mElmnt[id] == NULL)
	  {
		  Eprint(__FILE__, __LINE__);
		  fprintf(stderr, "class %s is not registered in elmlist.h.\n",
				  classname);
		  exit(1);
	  }

      mElmnt[id]->Init(objname, objnum, wfp);
      ignore(wfp, '\n');
    }
  } else {
    fprintf(stderr, "SCF Total data error: no [Components] section.\n");
    exit(1);
  }

  // read [Iterators] section
  /*

    [Iterators]
    # idnum objnum objname classname

    0 0 paddy Paddy  2
    1 0 river Storage 2

  */
  fscanf(wfp, "%s", header);
  if (strcmp(header, "[Iterators]") == 0) {
    for (int i = 0; i < mNumIte; i++) {
      int id;
      fscanf(wfp, "%d %s %d %s", &id, objname, &objnum, classname);
      if (id != i) {
        fprintf(stderr, "SCF Total data error in [Iterators] section. The id number %d was expected, but it was %d.\n", i, id);
        exit(1);
      }
      mIte[i] = new_element(classname);

	  // added by iYutaka (Thu Dec 28 11:50:19 2000)
	  // to check whether mIte[i] is NULL or not.
	  if (mIte[i] == NULL)
	  {
		  Eprint(__FILE__, __LINE__);
		  fprintf(stderr, "class %s is not registered in elmlist.h.\n",
				  classname);
		  exit(1);
	  }

      mIte[i]->Init(objname, objnum, wfp);
      ignore(wfp, '\n');
    }
  } else {
    fprintf(stderr, "SCF Total data error: no [Iterators] section.\n");
    exit(1);
  }

  // read [OutputPorts] section
  /*

    [OutputPorts]
    # objnum objname classname porttype

    0  filename  O_file  double

   */
  fscanf(wfp, "%s", header);
  if (strcmp(header, "[OutputPorts]") == 0) {
    for (int i = 0; i < mNumSps; i++) {
      int id, linelength;
      fscanf(wfp, "%d %s %d %s %s",
        &id, objname, &linelength, classname, porttype);
      if (id != i) {
        fprintf(stderr, "SCF Total data error in [OutputPorts] section. The id number %d was expected, but it was %d.\n", i, id);
        exit(1);
      }
      mSps[i] = new_port(classname, porttype);

	  // added by iYutaka (Thu Dec 28 11:50:19 2000)
	  // to check whether mSps[i] is NULL or not.
	  if (mSps[i] == NULL)
	  {
		  Eprint(__FILE__, __LINE__);
		  fprintf(stderr, "port %s (type %s) is not registered in portlist.h.\n",
				  classname, porttype);
		  exit(1);
	  }

      mSps[i]->Init(objname, linelength);
    }
  } else {
    fprintf(stderr, "SCF Total data error: no [OutputPorts] section.\n");
    exit(1);
  }

  fclose(wfp);
}

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
void ScfTotalSystem::Register_input_ports(void)
{
  for (int i = 0; i < mNumRps; i++) {
    Register((void*)mRps[i]);
  }
}

  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ¦üҤϿޤ饹ޤConstruct 
  // ƤФޤϿƤʤ¦üҤ³Ǥޤ.
  // </japanese>
void ScfTotalSystem::Register_output_ports(void)
{
  for (int i = 0; i < mNumSps; i++) {
    Register((void*)mSps[i]);
  }
}

////////////////////////////////////////////////////////////////////////
void ScfTotalSystem::Register_components(void)
{
  for (int i = 0; i < mNumElmnt; i++) {
    Register((void*)mElmnt[i]);
  }
}

////////////////////////////////////////////////////////////////////////
void ScfTotalSystem::Register_iterators(void)
{

  if (number_of_iterator_sets <= 0) return;

  FILE* wfp = dataFileOpen(mScffile);

  const int BUFFLEN = 256;
  char line[BUFFLEN];

  Boolean foundp = NO;
  while (fscanf(wfp, "%s", line) != EOF) {
    if (strcmp(line, "[RegisterIterators]") == 0) {
      foundp = YES; break;
    }
  }
  if (foundp == NO) { Eprint(__FILE__, __LINE__); exit(1); }

  for (int m = 0; m < number_of_iterator_sets; m++) {
    int num;
    fscanf(wfp, "%d", &num);

	//; iterator_array[i] == NULL ȤʤäƤ i õ. ʤä exit
	int i; Boolean flag = NO;
	for (i = 0; i < number_of_iterator_sets; i++) {
		if (iterator_array[i] == NULL) {
			flag = YES;
			break;
		}
	}
	if (flag == NO)
		{ Eprint(__FILE__, __LINE__); Oprint(); exit(1); }
			// Sub_system::Register(void*,...)
			// iterator_array is already full.
			// Please check "number_of_iterator_sets".

	for (int k = 0; k < num; k++) {
      int s;
      fscanf(wfp, "%d", &s);
      Register(mIte[s]);
	}

	if (Make_object_array(iterator_array[i], number_of_iterators[i])
		!= YES) {
		Eprint(__FILE__, __LINE__);
			// Sub_system::Register(int, ...)
			// Cannot convert object_list to "iterator_array".
		fprintf(stderr, "\t\t iterator_array[%d]\n", i);
		exit(1);
	}

	object_list->Clear();
  }
  fclose(wfp);
}

////////////////////////////////////////////////////////////////////////
void ScfTotalSystem::Connect_ports(void)
{
  FILE* wfp = dataFileOpen(mScffile);
  NEWCHECK(wfp);

  const int BUFFLEN = 256;
  char line[BUFFLEN];
  Boolean foundp = NO;
  while (fscanf(wfp, "%s", line) != EOF) {
    if (strcmp(line, "[ConnectPorts]") == 0) {
      foundp = YES; break;
    }
  }
  if (foundp == NO) { Eprint(__FILE__, __LINE__); exit(1); }

  char objname_s[81];  char objname_r[81];
  int objnum_s, objnum_r;
  while (1) {
    char c[10];
    fscanf(wfp, "%s", c);
    if (c[0] == 'z') { break; }
    // there are nine cases
    if (c[0] == 'r') {
      // the sender is a receive port
      int i;  fscanf(wfp, "%d", &i);  // receive port id
      // the receiver
      char r[10];  fscanf(wfp, "%s", r); int j;
      switch (r[0]) {
      case 'c':
        // the receiver is a usual component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect((void*)mRps[i], (void*)mElmnt[j], objname_r, objnum_r);
        break;
      case 'i':
        // the receiver is a iterator component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect(mRps[i], mIte[j], objname_r, objnum_r);
        break;
      case 's':
        // the receiver is a send port
        fscanf(wfp, "%d", &j);
        Connect(mRps[i], mSps[j]);
        break;
      default:
        Eprint(__FILE__, __LINE__); exit(1);
      }
    }
    else if (c[0] == 'c') {
      // the sender is a usual component
      int i;
      fscanf(wfp, "%d %s %d", &i, objname_s, &objnum_s);
      // the receiver
      char r[10];  fscanf(wfp, "%s", r); int j;
      switch (r[0]) {
      case 'c':
        // the receiver is a usual component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect((void*)mElmnt[i], objname_s, objnum_s,
          (void*)mElmnt[j], objname_r, objnum_r);
        break;
      case 'i':
        // the receiver is a iterator component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect((void*)mElmnt[i], objname_s, objnum_s,
          (void*)mIte[j], objname_r, objnum_r);
        break;
      case 's':
        // the receiver is a send port
        fscanf(wfp, "%d", &j);
        Connect((void*)mElmnt[i], objname_s, objnum_s, (void*)mSps[j]);
        break;
      default:
        Eprint(__FILE__, __LINE__); exit(1);
      }
    }
    else if (c[0] == 'i') {
      // the sender is a iterator component
      int i;
      fscanf(wfp, "%d %s %d", &i, objname_s, &objnum_s);
      // the receiver
      char r[10];  fscanf(wfp, "%s", r); int j;
      switch (r[0]) {
      case 'c':
        // the receiver is a usual component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect((void*)mIte[i], objname_s, objnum_s,
          (void*)mElmnt[j], objname_r, objnum_r);
        break;
      case 'i':
        // the receiver is a iterator component
        fscanf(wfp, "%d %s %d", &j, objname_r, &objnum_r);
        Connect((void*)mIte[i], objname_s, objnum_s,
          (void*)mIte[j], objname_r, objnum_r);
        break;
      case 's':
        // the receiver is a send port
        fscanf(wfp, "%d", &j);
        Connect((void*)mIte[i], objname_s, objnum_s, (void*)mSps[j]);
        break;
      default:
        Eprint(__FILE__, __LINE__); exit(1);
      }
    }
    else {
      Eprint(__FILE__, __LINE__); exit(1);
    }
  } // while loop
  fclose(wfp);
}

////////////////////////////////////////////////////////////////////////
// <japanese>
// ľ̿³
//
//   [ConnectComponents]
//   c 2 c 3 0 # mElmnt[2]->Register((void*)mElmnt[3], 0)
//   i 0 i 1 2 # mIte[0]->Register((void*)mIte[1], 2);
//   z # end mark
//
// </japanese>
void ScfTotalSystem::Connect_components(void)
{
  FILE* wfp = dataFileOpen(mScffile);
  NEWCHECK(wfp);

  const int BUFFLEN = 256;
  char line[BUFFLEN];
  Boolean foundp = NO;
  while (fscanf(wfp, "%s", line) != EOF) {
    if (strcmp(line, "[ConnectComponents]") == 0) {
      foundp = YES; break;
    }
  }
  if (foundp == NO) { Eprint(__FILE__, __LINE__); exit(1); }

  while (1) {
    char c1[10], c2[10];
    int  i1, i2, j;
    fscanf(wfp, "%s", c1);
    if (c1[0] == 'z') { break; }
    fscanf(wfp, "%d %s %d %d", &i1, c2, &i2, &j);
    Element *p1, *p2;
    p1 = (c1[0] == 'c')? mElmnt[i1] : mIte[i1];
    p2 = (c2[0] == 'c')? mElmnt[i2] : mIte[i2];
    p1->Register((void*)p2, j);
  }
  fclose(wfp);
}

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
